/**
 * Copyright (c) 2010 Perforce Software.  All rights reserved.
 */
package com.perforce.p4scout;

import java.util.List;

import android.content.Context;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.widget.BaseAdapter;
import android.widget.ImageView;
import android.widget.LinearLayout;
import android.widget.TextView;

import com.perforce.p4java.core.IChangelist;
import com.perforce.p4java.core.IJob;
import com.perforce.p4java.core.file.IFileSpec;

/**
 * The Class ChangeDetailAdapter.
 */
public class ChangeDetailAdapter extends BaseAdapter {

	/**
     * Types of Items.
     */
    public enum ItemType {
	    /** FILEREV - A file revision type item. */
	    FILEREV, 
		/** JOB - A job type item. */
		JOB, 
		/** OTHER - An item that is not a file revision or a job. */
		OTHER;
    }
	protected IChangelist changeData;
	protected List<IFileSpec> fileCache;
	protected List<IJob> jobCache;
	protected int jobs = 0;
    protected int files = 0;
    protected boolean oldError = false;
    protected boolean filesError = false;
    protected boolean jobsError = false;
    protected Context context;
	
    protected LayoutInflater inflater;
    
    /**
     * Instantiates a new change detail adapter.
     *
     * @param context the context. (required)
     */
    public ChangeDetailAdapter(Context context) {
		this.context = context;
		this.inflater = (LayoutInflater) context
				.getSystemService(Context.LAYOUT_INFLATER_SERVICE);
	}
    
	/**
	 * Instantiates a new change detail adapter.
	 *
	 * @param context the context. (required)
	 * @param changeData the changelist information. (required)
	 */
	public ChangeDetailAdapter(Context context, IChangelist changeData) {
		this.changeData = changeData;
		this.context = context;
		this.inflater = (LayoutInflater) context
				.getSystemService(Context.LAYOUT_INFLATER_SERVICE);
	}

	/**
	 * Adds the changelist information.
	 *
	 * @param changeData the changelist information. (required)
	 */
	public void addChangeDetails(IChangelist changeData) {
		if (this.changeData == null) {
			this.changeData = changeData;
		}
	}
	
	/**
	 * Adds the file specs.
	 *
	 * @param fileCache a List of FileSpecs related to the changgelist. (required)
	 */
	public void addFileSpecs(List<IFileSpec> fileCache) {
		if (this.fileCache == null) {
			this.fileCache = fileCache;
			if (fileCache.size() == 0)  {
				// There are no associated files, so we show the header + a placeholder
				files = 2;
			} else {
				// There are associated files, but we also need to show the header
				files = fileCache.size() + 1;
			}
		}
	}
	
	/**
	 * Adds the jobs.
	 *
	 * @param jobCache a List of associated jobs
	 */
	public void addJobs(List<IJob> jobCache) {
		if (this.jobCache == null) {
			this.jobCache = jobCache;
			if (jobCache.size() == 0)  {
				// There are no associated jobs, so we show the header + a placeholder
				jobs = 2;
			} else {
				// There are associated jobs, but we also need to show the header
				jobs = jobCache.size() + 1;
			}
		}
	}
	
	/**
	 * Sets the error flag for the jobs list.
	 */
	public void tooManyJobs() {
		jobCache = null;
		jobsError = true;
		jobs = 2;
	}
	
	/**
	 * Sets the error flag for the file revisions list.
	 */
	public void tooManyFiles() {
		fileCache = null;
		filesError = true;
		files = 2;
	}
	
	/**
	 * Returns the number of items in the list.
	 * <ul>
	 * <li>1 for the changelist description</li>
	 * <li>2 or more associated file revisions</li>
	 * <li>2 or more associated jobs</li>
	 * </ul>
	 * 
	 * @see android.widget.Adapter#getCount()
	 */
	@Override
	public int getCount() {
		if (changeData != null) {
			return 1 + files + jobs;
		}
		return 0;
	}

	/**
	 * gets the file revision item at the requested position.
	 * 
	 * @param position the position of the requested file revision. (required)
	 * @return the IFileSpec of the file revision at the specified position or null
	 */
	public IFileSpec getFilerev(int position) {
		if (fileCache != null && fileCache.size() > 0 && position > 0 &&
				position - 1 != 0 && position - 1 < files) {
			return fileCache.get(position - 2);
		}
		return null;
	}

	/**
	 * Gets the type of the item at the requested position.
	 * 
	 * @param position the position of the requested item. (required)
	 * @return the ItemType of the list item at the specified position.
	 * 
	 * @see android.widget.Adapter#getItem(int)
	 */
	@Override
	public ItemType getItem(int position) {
		if (fileCache != null && fileCache.size() > 0 && position > 0 && position - 1 != 0 &&
				position - 1 < files) {
			return ItemType.FILEREV;
		}
		if (jobCache != null && jobCache.size() > 0 && position > 0 &&
				position - (files + 1) != 0 && position - (files + 1) < jobs) {
			return ItemType.JOB;
		}
		return ItemType.OTHER;
	}
	
	/**
	 * @see android.widget.Adapter#getItemId(int)
	 */
	@Override
	public long getItemId(int position) {
		return position;
	}

	/**
	 * gets the job item at the requested position.
	 * 
	 * @param position the position of the requested job. (required)
	 * @return the job at the specified position or null
	 */
	public IJob getJob(int position) {
		if (jobCache != null && jobCache.size() > 0 && position > 0 &&
				position - (files + 1) != 0 && position - (files + 1) < jobs) {
			return jobCache.get(position - (files + 2));
		}
		return null;
	}

	/**
	 * This is the method that draws each part of the changelist description.<br/>
	 * We change how we draw the items based on their type.
	 * 
	 * @param position The position of the list item to be drawn 
	 * @param convertView 
	 * @param parent
	 * 
	 * @see android.widget.Adapter#getView(int, android.view.View, android.view.ViewGroup)
	 */
    public View getView(int position, View convertView, ViewGroup parent) {
    	
		if (position == 0) {
			// 0 will always be the changelist description
			if (convertView == null ||  convertView.findViewById(R.id.llChangeDetail) == null) {
			    convertView = inflater.inflate(R.layout.changedetaillisting, parent, false);
			}

			((LinearLayout) convertView.findViewById(R.id.llChangeDetails))
				.setVisibility(View.VISIBLE);
			((TextView) convertView.findViewById(R.id.tvFilesHeader))
				.setVisibility(View.GONE);
			((TextView) convertView.findViewById(R.id.tvNoFileRevs))
				.setVisibility(View.GONE);
			((TextView) convertView.findViewById(R.id.tvTooManyFiles))
				.setVisibility(View.GONE);
			((TextView) convertView.findViewById(R.id.tvJobsHeader))
				.setVisibility(View.GONE);
			((TextView) convertView.findViewById(R.id.tvNoJobsAttached))
				.setVisibility(View.GONE);
			((TextView) convertView.findViewById(R.id.tvTooManyJobs))
				.setVisibility(View.GONE);
			
			((TextView) convertView.findViewById(R.id.tvChangeNum))
				.setText("#" + Integer.toString(changeData.getId()));
			((TextView) convertView.findViewById(R.id.tvChangeTime))
				.setText(changeData.getDate().toLocaleString());
			((TextView) convertView.findViewById(R.id.tvChangeUser))
				.setText(changeData.getUsername());
			((TextView) convertView.findViewById(R.id.tvChangeDesc))
				.setText(changeData.getDescription());
			
		} else if (position - 1 < files) {
			// This must be in the files section
			if (position - 1 == 0) {
				// Show the files header
				if (convertView == null ||  convertView.findViewById(R.id.llChangeDetail) == null) {
				    convertView = inflater.inflate(R.layout.changedetaillisting, parent, false);
				}
				
				((LinearLayout) convertView.findViewById(R.id.llChangeDetails))
					.setVisibility(View.GONE);
				((TextView) convertView.findViewById(R.id.tvFilesHeader))
					.setVisibility(View.VISIBLE);
				((TextView) convertView.findViewById(R.id.tvNoFileRevs))
					.setVisibility(View.GONE);
				((TextView) convertView.findViewById(R.id.tvTooManyFiles))
					.setVisibility(View.GONE);
				((TextView) convertView.findViewById(R.id.tvJobsHeader))
					.setVisibility(View.GONE);
				((TextView) convertView.findViewById(R.id.tvNoJobsAttached))
					.setVisibility(View.GONE);
				((TextView) convertView.findViewById(R.id.tvTooManyJobs))
					.setVisibility(View.GONE);
				
			} else if (filesError == true || oldError == true) {
				// Show the too many files message
				if (convertView == null ||  convertView.findViewById(R.id.llChangeDetail) == null) {
				    convertView = inflater.inflate(R.layout.changedetaillisting, parent, false);
				}
				
				((LinearLayout) convertView.findViewById(R.id.llChangeDetails))
					.setVisibility(View.GONE);
				((TextView) convertView.findViewById(R.id.tvFilesHeader))
					.setVisibility(View.GONE);
				((TextView) convertView.findViewById(R.id.tvNoFileRevs))
					.setVisibility(View.GONE);
				((TextView) convertView.findViewById(R.id.tvTooManyFiles))
					.setVisibility(View.VISIBLE);
				((TextView) convertView.findViewById(R.id.tvJobsHeader))
					.setVisibility(View.GONE);
				((TextView) convertView.findViewById(R.id.tvNoJobsAttached))
					.setVisibility(View.GONE);
				((TextView) convertView.findViewById(R.id.tvTooManyJobs))
					.setVisibility(View.GONE);
				
				if (oldError == true) {
					((TextView) convertView.findViewById(R.id.tvTooManyFiles))
						.setText(R.string.tvServerTooOld);
				}
				
			} else if (fileCache.size() == 0) {
				// Show the "no associated files" placeholder
				if (convertView == null ||  convertView.findViewById(R.id.llChangeDetail) == null) {
				    convertView = inflater.inflate(R.layout.changedetaillisting, parent, false);
				}
				
				((LinearLayout) convertView.findViewById(R.id.llChangeDetails))
					.setVisibility(View.GONE);
				((TextView) convertView.findViewById(R.id.tvFilesHeader))
					.setVisibility(View.GONE);
				((TextView) convertView.findViewById(R.id.tvNoFileRevs))
					.setVisibility(View.VISIBLE);
				((TextView) convertView.findViewById(R.id.tvTooManyFiles))
					.setVisibility(View.GONE);
				((TextView) convertView.findViewById(R.id.tvJobsHeader))
					.setVisibility(View.GONE);
				((TextView) convertView.findViewById(R.id.tvNoJobsAttached))
					.setVisibility(View.GONE);
				((TextView) convertView.findViewById(R.id.tvTooManyJobs))
					.setVisibility(View.GONE);
				
			} else {
				// Show the file revision listing
				if (convertView == null ||  convertView.findViewById(R.id.tvDepotPath) == null) {
				    convertView = inflater.inflate(R.layout.filelisting, parent, false);
				}
				
				((TextView) convertView.findViewById(R.id.tvDepotPath))
					.setText(fileCache.get(position - 2).getDepotPath() + "#" +
						Integer.toString(fileCache.get(position - 2).getEndRevision()));
				
				if (fileCache.get(position - 2).getAction() != null) {
					((TextView) convertView.findViewById(R.id.tvChangeType))
						.setText(fileCache.get(position - 2).getAction().name());
					
					switch (fileCache.get(position - 2).getAction()) {
					case ADD:
						((ImageView) convertView.findViewById(R.id.ivAction))
							.setImageResource(R.drawable.revisionadd);
						break;
					case EDIT:
						((ImageView) convertView.findViewById(R.id.ivAction))
							.setImageResource(R.drawable.revisionedit);
						break;
					case DELETE:
						((ImageView) convertView.findViewById(R.id.ivAction))
							.setImageResource(R.drawable.revisiondelete);
						break;
					case INTEGRATE:
						((ImageView) convertView.findViewById(R.id.ivAction))
							.setImageResource(R.drawable.revisionintegrate);
						break;
					case MOVE_ADD:
						((ImageView) convertView.findViewById(R.id.ivAction))
							.setImageResource(R.drawable.revisionmoveadd);
						break;
					case MOVE_DELETE:
						((ImageView) convertView.findViewById(R.id.ivAction))
							.setImageResource(R.drawable.revisionmovedelete);
						break;
					case BRANCH:
						((ImageView) convertView.findViewById(R.id.ivAction))
							.setImageResource(R.drawable.revisionbranch);
						break;
					}
				} else {
					// Workaround for the import bug
					((TextView) convertView.findViewById(R.id.tvChangeType))
						.setText("IMPORT");
					((ImageView) convertView.findViewById(R.id.ivAction))
						.setImageResource(R.drawable.revisionbranch);
				}
			}
		} else if (position - (files + 1) < jobs) {
			// This must be in the jobs section
			if (position - (files + 1) == 0) {
				// Show the jobs header
				if (convertView == null ||  convertView.findViewById(R.id.llChangeDetail) == null) {
				    convertView = inflater.inflate(R.layout.changedetaillisting, parent, false);
				}
				
				((LinearLayout) convertView.findViewById(R.id.llChangeDetails))
					.setVisibility(View.GONE);
				((TextView) convertView.findViewById(R.id.tvFilesHeader))
					.setVisibility(View.GONE);
				((TextView) convertView.findViewById(R.id.tvNoFileRevs))
					.setVisibility(View.GONE);
				((TextView) convertView.findViewById(R.id.tvTooManyFiles))
					.setVisibility(View.GONE);
				((TextView) convertView.findViewById(R.id.tvJobsHeader))
					.setVisibility(View.VISIBLE);
				((TextView) convertView.findViewById(R.id.tvNoJobsAttached))
					.setVisibility(View.GONE);
				((TextView) convertView.findViewById(R.id.tvTooManyJobs))
					.setVisibility(View.GONE);
				
			} else if (jobsError == true) {
				// Show the too many files message
				if (convertView == null ||  convertView.findViewById(R.id.llChangeDetail) == null) {
				    convertView = inflater.inflate(R.layout.changedetaillisting, parent, false);
				}
				
				((LinearLayout) convertView.findViewById(R.id.llChangeDetails))
					.setVisibility(View.GONE);
				((TextView) convertView.findViewById(R.id.tvFilesHeader))
					.setVisibility(View.GONE);
				((TextView) convertView.findViewById(R.id.tvNoFileRevs))
					.setVisibility(View.GONE);
				((TextView) convertView.findViewById(R.id.tvTooManyFiles))
					.setVisibility(View.GONE);
				((TextView) convertView.findViewById(R.id.tvJobsHeader))
					.setVisibility(View.GONE);
				((TextView) convertView.findViewById(R.id.tvNoJobsAttached))
					.setVisibility(View.GONE);
				((TextView) convertView.findViewById(R.id.tvTooManyJobs))
					.setVisibility(View.VISIBLE);
				
			} else if (jobCache.size() == 0) {
				// Show the "no associated jobs" placeholder
				if (convertView == null ||  convertView.findViewById(R.id.llChangeDetail) == null) {
				    convertView = inflater.inflate(R.layout.changedetaillisting, parent, false);
				}
				
				((LinearLayout) convertView.findViewById(R.id.llChangeDetails))
					.setVisibility(View.GONE);
				((TextView) convertView.findViewById(R.id.tvFilesHeader))
					.setVisibility(View.GONE);
				((TextView) convertView.findViewById(R.id.tvNoFileRevs))
					.setVisibility(View.GONE);
				((TextView) convertView.findViewById(R.id.tvTooManyFiles))
					.setVisibility(View.GONE);
				((TextView) convertView.findViewById(R.id.tvJobsHeader))
					.setVisibility(View.GONE);
				((TextView) convertView.findViewById(R.id.tvNoJobsAttached))
					.setVisibility(View.VISIBLE);
				((TextView) convertView.findViewById(R.id.tvTooManyJobs))
					.setVisibility(View.GONE);
				
			} else {
				// Show the job listing
				if (convertView == null ||  convertView.findViewById(R.id.tvJobName) == null) {
				    convertView = inflater.inflate(R.layout.joblisting, parent, false);
				}
				
				((TextView) convertView.findViewById(R.id.tvJobName))
					.setText(jobCache.get(position - (files + 2)).getId());
				((TextView) convertView.findViewById(R.id.tvJobStatus))
					.setText((String) jobCache.get(position - (files + 2))
							.getRawFields().get("status"));
				((TextView) convertView.findViewById(R.id.tvShortDesc))
					.setText(jobCache.get(position - (files + 2)).getDescription().split("\n")[0]);
			}
			
		}
		
		return convertView;
	}

	public void serverTooOld() {
		fileCache = null;
		oldError = true;
		files = 2;
	}

}
