/**
 * Copyright (c) 2010 Perforce Software.  All rights reserved.
 */
package com.perforce.p4scout;

import static com.perforce.p4scout.Constants.CALLER;
import static com.perforce.p4scout.Constants.CHANGENUM;
import static com.perforce.p4scout.Constants.ERROR;
import static com.perforce.p4scout.Constants.ID;
import static com.perforce.p4scout.Constants.PASSWORDINVALID;
import static com.perforce.p4scout.Constants.PASSWORDNOTSET;
import static com.perforce.p4scout.Constants.SESSIONLOGGEDOUT;

import java.util.List;

import android.app.Activity;
import android.content.Context;
import android.content.Intent;
import android.content.SharedPreferences;
import android.content.res.Configuration;
import android.os.Bundle;
import android.os.Handler;
import android.os.Looper;
import android.preference.PreferenceManager;
import android.view.View;
import android.widget.AdapterView;
import android.widget.ListView;
import android.widget.Toast;
import android.widget.AdapterView.OnItemClickListener;

import com.perforce.p4java.core.IChangelistSummary;

/**
 * This class displays a list of recently submitted changelist from a Perforce Server.<br/>
 * These are Perforce Server details are passed as an extra:<p>
 * 
 * <ul><li>string ID The ServerId from the P4SERVERS table (required)</ul>
 * 
 * The number of changelists to fetch is defined in the Shared Property: prefGetChangelists
 */
public class ChangesList extends Activity {

	protected ChangesListAdapter changesAdapter;
	protected String serverId;
	protected PerforceSession serverCaller;
	protected Context context;
	private Thread[] runner;

	/** The Handler for displaying errors */
	private Handler errorHandle;
	
	/** The Handler for updating the Changelist details in the View */
	private Handler listUpdateHandle = new Handler();
	
	/**
     * The Runnable for updating the list of changelists in the View.
     * 
     * @see com.perforce.p4scout.ChangesList#getChanges(int, String, String, int)
     */
	private Runnable listUpdateUpdater = new Runnable() {
		public void run() {
			setTitle(getString(R.string.strLastSpc) + " " +
					Integer.toString(changesAdapter.getCount()-1) + " " +
					getString(R.string.strSpcChangesSpc) + " - " + serverCaller.getServerName() );
			changesAdapter.notifyDataSetChanged();
		}
	};
	
	/**
	 * Gets the next X number of changelists and calls the update Handler.
	 *
	 * @param changesToGet the number changes to fetch (required)
	 * @param offset the number of changes to offset the fetch by (required)
	 * 
	 * @return the changes@see com.perforce.p4scout.ChangesList#listUpdateHandle
	 */
	private void getChanges(final int changesToGet, final int offset) {
	class Worker extends Thread {
	    @Override
		public void run() {
			try {
				serverCaller.connect(true);
				if (Thread.interrupted()) {
					return;
				}
			    List<IChangelistSummary> changes = serverCaller.getChangelists(changesToGet + offset);
			    if (Thread.interrupted()) {
					return;
				}
				for (int j=offset; j < changes.size(); j++) {
					changesAdapter.addChange(changes.get(j));
				}
				changesAdapter.setLoading(false);
				if (Thread.interrupted()) {
					return;
				}
			    listUpdateHandle.post(listUpdateUpdater);
			    serverCaller.disconnect();
			    
			} catch (final Exception e) {
				if (Thread.interrupted()) {
					return;
				}
				errorHandle.post(new Runnable() {
					@Override
					public void run() {
						Toast.makeText(getApplicationContext(), e.getMessage().trim(),
								Toast.LENGTH_LONG).show();
						if (e.getMessage().trim().contains(PASSWORDINVALID) || 
								e.getMessage().trim().contains(SESSIONLOGGEDOUT) ||
				    			e.getMessage().trim().contains(PASSWORDNOTSET)) {
							Intent loginIntent = new Intent(context, LoginDialog.class);
							loginIntent.putExtra(ID, serverId);
							loginIntent.putExtra(ERROR, e.getMessage());
							loginIntent.putExtra(CALLER, getLocalClassName());
				    		startActivity(loginIntent);
						}
					}
				});
	    		    		
				e.printStackTrace();
				finish();
			} 
	    }
	}
	runner[0] = new Worker();
	runner[0].start();
}

	/**
	 * This is the initialisation method for the recent changelists view.<br/>
	 * This displays the view and kicks off the Perforce commands in a new thread.
	 * 
	 * @see android.app.Activity#onCreate(android.os.Bundle)
	 */
	@Override
	protected void onCreate(Bundle savedInstanceState)
	{
		super.onCreate(savedInstanceState);
		setContentView(R.layout.changeslist);
		context = this;
		errorHandle = new Handler(Looper.getMainLooper());
		runner = new Thread[1];
		Intent i = getIntent();
		Bundle extras = i.getExtras();
		serverId = (String) extras.get(ID);
		SharedPreferences sp = PreferenceManager.getDefaultSharedPreferences(this);
		final int changesToGet = Integer.parseInt(sp.getString("prefGetChangelists", "10"));
		
		OnItemClickListener changeListClickListener = new OnItemClickListener() {
			public void onItemClick(AdapterView<?> parent, View view, int position, long id) {
				IChangelistSummary change = changesAdapter.getItem(position);
				if (change == null && !changesAdapter.getloading()) {
					changesAdapter.setLoading(true);
					changesAdapter.notifyDataSetChanged();
					getChanges(changesToGet, changesAdapter.getCount()-1);
				} else if (change == null) {
					//We're still loading, so we'll ignore this!
				} else {
					Intent i = new Intent(view.getContext(), ChangeDetail.class);
					i.putExtra(CHANGENUM, change.getId());
					i.putExtra(ID, serverId);
					startActivity(i);
				}
			}
		};
		
		changesAdapter = new ChangesListAdapter(this);
		ListView changes = (ListView) findViewById(R.id.lvChanges);
		changes.setAdapter(changesAdapter);
		changes.setOnItemClickListener(changeListClickListener);
		
		serverCaller = new PerforceSession(this, serverId);
	    
	    setTitle(getString(R.string.strLastSpc) + " " +
	    		Integer.toString(changesAdapter.getCount()-1) + " " +
	    		getString(R.string.strSpcChangesSpc) + " - " + serverCaller.getServerName() );
		getChanges(changesToGet, 0);	    
	}
	
	/**
     * This kills all background threads when we exit. There is no need for them to consume
     * resources when we are doing something else.
     */
	@Override
    protected void onPause() {
    	super.onPause();
    	if (runner == null) {
    		return;
    	}
    	for (int i=0; i<runner.length; i++) {
    		if (runner[i] != null) {
    			runner[i].interrupt();
    		}
    	}
    }
	
	/**
	 * This prevents the activity from being restarted when the orientation changes
	 */
    @Override
    public void onConfigurationChanged(Configuration newConfig){        
        super.onConfigurationChanged(newConfig);
    }
}
