/**
 * Copyright (c) 2010 Perforce Software.  All rights reserved.
 */
package com.perforce.p4scout;

import java.util.LinkedList;
import java.util.List;

import android.content.Context;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.widget.BaseAdapter;
import android.widget.LinearLayout;
import android.widget.TextView;

import com.perforce.p4java.core.IChangelistSummary;

/**
 * This is the ListAdaptor implementation for displaying lists of Perforce Changes.
 * 
 * @see android.widget.BaseAdapter
 */
public class ChangesListAdapter extends BaseAdapter {

	protected List<IChangelistSummary> changes = new LinkedList<IChangelistSummary>();
    protected LayoutInflater inflater;
    protected Context context;
    protected boolean loading = true;
    
    /**
     * Instantiates a new list adapter for Changelists.
     *
     * @param context parent class' context
     */
    public ChangesListAdapter(Context context) {
		this.context = context;
		this.inflater = (LayoutInflater) context
				.getSystemService(Context.LAYOUT_INFLATER_SERVICE);
    }   
    
    /**
     * Adds a changelist to the list of changelists.
     *
     * @param change a changelist summary object (required)
     */
    public void addChange(IChangelistSummary change) {
    	changes.add(change);
    }
    
	/**
	 * This returns the number of items in the list of changelists plus the final item for fetching
	 * more.
	 * 
	 * @see android.widget.Adapter#getCount()
	 */
	public int getCount() {
		return changes.size() + 1;
	}
	
	/**
	 * This returns the changelist object that the specified position.
	 * 
	 * @param position The item in the list of changelists to return (required)
	 * @return the changelist summary object, or null if it's the non-changelist final item.
	 * 
	 * @see android.widget.Adapter#getItem(int)
	 */
	public IChangelistSummary getItem(int position) {
		if (position >= changes.size())
			return null;
		return changes.get(position);
	}
	
	/* (non-Javadoc)
	 * @see android.widget.Adapter#getItemId(int)
	 */
	public long getItemId(int position) {
		return position;
	}
	
	/**
	 * Gets the loading flag.
	 *
	 * @return whethere or not we are loading
	 */
	public boolean getloading() {
		return loading;
	}

	/** 
	 * This is the method that draws each changelist item in the list of changelists.<br/>
	 * We change how we draw the last item based on whether more data is being fetched.
	 * 
	 * @param position The position of the list item to be drawn 
	 * @param convertView 
	 * @param parent
	 * 
	 * @see android.widget.Adapter#getView(int, android.view.View, android.view.ViewGroup)
	 */
	public View getView(int position, View convertView, ViewGroup parent) {
		if (convertView == null) {
		    convertView = inflater.inflate(R.layout.changelisting, parent, false);
		}
		
		if (position >= changes.size()) {
			// The requested view out of the range of gathered changelists.
			if (loading) {
				// Show the loading progress view
				((LinearLayout) convertView.findViewById(R.id.llNext))
					.setVisibility(View.GONE);
				((LinearLayout) convertView.findViewById(R.id.llChangeBit))
					.setVisibility(View.GONE);
				((LinearLayout) convertView.findViewById(R.id.llLoading))
					.setVisibility(View.VISIBLE);
			} else {
				// Show the "get next lot" button
				((LinearLayout) convertView.findViewById(R.id.llChangeBit))
					.setVisibility(View.GONE);
				((LinearLayout) convertView.findViewById(R.id.llLoading))
					.setVisibility(View.GONE);
				((LinearLayout) convertView.findViewById(R.id.llNext))
					.setVisibility(View.VISIBLE);
			}
		} else {
			// Populate the normal view
			((LinearLayout) convertView.findViewById(R.id.llLoading))
				.setVisibility(View.GONE);
			((LinearLayout) convertView.findViewById(R.id.llNext))
				.setVisibility(View.GONE);
			((LinearLayout) convertView.findViewById(R.id.llChangeBit))
				.setVisibility(View.VISIBLE);					
			((TextView) convertView.findViewById(R.id.tvChangeNum))
				.setText(Integer.toString(changes.get(position).getId()));
			((TextView) convertView.findViewById(R.id.tvChangeTime))
				.setText(changes.get(position).getDate().toLocaleString());
			((TextView) convertView.findViewById(R.id.tvShortDesc))
				.setText(changes.get(position).getDescription().split("\n")[0]);
		}
		
		return convertView;
	}

	/**
	 * The refresher method.
	 */
	public void refresh() {
    	notifyDataSetChanged();
    }

	/**
	 * Sets the loading flag. This will cause the last item in the list to inform the user that the
	 * list is being updated.
	 *
	 * @param loading is the dataset being fetched? (required)
	 */
	public void setLoading(boolean loading) {
		this.loading = loading;
	}

}
