/**
 * Copyright (c) 2010 Perforce Software.  All rights reserved.
 */
package com.perforce.p4scout;

import java.util.LinkedList;
import java.util.List;

import android.content.Context;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.widget.BaseAdapter;
import android.widget.TextView;

/**
 * HelpAdapter is a BaseAdapter for displaying help topics in a listview.
 * The topics are stored as a List of 2-element arrays.
 * <ul>
 * <li>[0] stores the R.string id for the topic title.</li>
 * <li>[1] stores the R.layout id for the topic article</li>
 * </ul>
 */
public class HelpAdapter extends BaseAdapter {

	protected List<int[]> helpTopics = new LinkedList<int[]>();
	protected int selected = -1;
	protected LayoutInflater inflater;
    protected Context context;
    
    /**
     * Instantiates a new help adapter.
     *
     * @param context the context. (required)
     */
    public HelpAdapter(Context context) {
		this.context = context;
		this.inflater = (LayoutInflater) context
				.getSystemService(Context.LAYOUT_INFLATER_SERVICE);
    }
    
    /**
     * Adds a help topic to the List.
     *
     * @param name R.string id of the topic title. (required)
     * @param view R.layout if of the topic article. (required)
     */
    public void addTopic(int name, int view) {
    	helpTopics.add(new int[] { name, view } );
    }
	
    /**
	 * Returns the number of items in the list.<br/>
	 * If a topic is selected, we need to include the article in the count.
	 *  
	 * @see android.widget.Adapter#getCount()
	 */
	@Override
	public int getCount() {
		if (selected == -1) {
			return helpTopics.size();
		} else {
			return helpTopics.size() + 1;
		}
	}
    
	/**
	 * Returns the array of ids for the topic at the specified position
	 * 
	 * @see android.widget.Adapter#getItem(int)
	 */
	@Override
	public int[] getItem(int position) {
		if (selected != -1 && position - 1 >= selected) {
    		// A topic is selected and this position is after/or the expanded topic, so to get
    		// back to actual topic numbers we need to subtract 1.
    		position--;
    	}
		return helpTopics.get(position);
	}

	/**
	 * @see android.widget.Adapter#getItemId(int)
	 */
	@Override
	public long getItemId(int pos) {
		return pos;
	}

	/**
	 * This is the method that draws each help topic in a list.<br/>
	 * If a topic is selected, we insert the article view below that topic.
	 * 
	 * @param position The position of the list item to be drawn 
	 * @param convertView 
	 * @param parent
	 * 
	 * @see android.widget.Adapter#getView(int, android.view.View, android.view.ViewGroup)
	 */
	public View getView(int position, View convertView, ViewGroup parent) {
		if (selected == -1 || position - 1 != selected) {
			if (convertView == null || convertView.findViewById(R.id.tvHelpTopicName) == null) {
				convertView = inflater.inflate(R.layout.helpitem, parent, false);
			}
			if (selected >= 0 && position - 1 > selected) {
				// This position is after the expanded topic, so to get back to actual topic 
	    		// numbers we need to subtract 1.
				position--;
			}
			// Display the topic header
			((TextView) convertView.findViewById(R.id.tvHelpTopicName))
				.setText(helpTopics.get(position)[0]);
		
		} else if (position - 1 == selected) {
			// The previous topic is selected, so we display the article here instead
			convertView = inflater.inflate(helpTopics.get(selected)[1], parent, false);
		}
		
		return convertView;
	}

	/**
     * Mark a topic as selected. This topic's article will be displayed
     *
     * @param position the position of the article. (required)
     */
    public void selectTopic(int position) {
    	if (selected != -1) {
    		// A topic is selected
    		if (position - 1 == selected) {
    			// This is the topic's article, and cannot be selected
    			return;
    		}
	    	if (position - 1 > selected) {
	    		// This position is after the expanded topic, so to get back to actual topic 
	    		// numbers we need to subtract 1.
	    		position--;
	    	}
    	}
    	if (position == selected) {
    		// The selected topic is selected again, so we hide the article
    		selected = -1;
    	} else {
    		// A new topic is selected, so we'll select it.
    		selected = position;
    	}
    }

}
