/**
 * Copyright (c) 2010 Perforce Software.  All rights reserved.
 */
package com.perforce.p4scout;

import static com.perforce.p4scout.Constants.CALLER;
import static com.perforce.p4scout.Constants.CHANGENUM;
import static com.perforce.p4scout.Constants.ERROR;
import static com.perforce.p4scout.Constants.ID;
import static com.perforce.p4scout.Constants.JOBID;
import static com.perforce.p4scout.Constants.PASSWORDINVALID;
import static com.perforce.p4scout.Constants.PASSWORDNOTSET;
import static com.perforce.p4scout.Constants.SESSIONLOGGEDOUT;

import java.util.List;

import android.app.Activity;
import android.app.ProgressDialog;
import android.content.Context;
import android.content.Intent;
import android.content.res.Configuration;
import android.os.Bundle;
import android.os.Handler;
import android.os.Looper;
import android.view.View;
import android.view.Window;
import android.widget.AdapterView;
import android.widget.ListView;
import android.widget.Toast;
import android.widget.AdapterView.OnItemClickListener;

import com.perforce.p4java.core.IFix;
import com.perforce.p4java.core.IJob;
import com.perforce.p4java.core.IJobSpec;
import com.perforce.p4java.exception.NoSuchObjectException;
import com.perforce.p4scout.JobDetailAdapter.ItemType;

/**
 * This class displays detailed information about the specified Job. Extras:
 * 
 * <ul><li>ID The ServerId from the P4SERVERS table (required)
 * <li>int JOBID The job name number (required)</ul>
 */
public class JobDetail extends Activity {
	
	protected Context context;
	protected PerforceSession serverCaller;
	protected IJob jobData;
	protected IJobSpec jobSpec;
	protected List<IFix> jobFixes;
	protected String serverId;
	protected ProgressDialog loading;
	protected JobDetailAdapter jobAdapter;
	protected boolean fixesError = false;
	private Thread[] runner;

	/** The Handler for updating the job details in the View */
	private Handler errorHandle;
	
	/** The Handler for updating the job details in the View */
	private Handler detailUpdateHandle = new Handler();
    
	/**
     * The Runnable for updating the job details in the View.
     * This requires jobData, jobSpec and jobFixes to be populated.
     * 
     * @see com.perforce.p4scout.JobDetail#getJob(String, String, String)
     */
    private Runnable detailUpdateUpdater = new Runnable() {
		public void run() {
			if (jobSpec != null && jobData != null) {
				jobAdapter.addJobDetails(jobData, jobSpec);
				jobAdapter.notifyDataSetChanged();
				loading.dismiss();
			}
			if (jobFixes != null && fixesError == false) {
				jobAdapter.addFixes(jobFixes);
				jobAdapter.notifyDataSetChanged();
				setProgressBarIndeterminateVisibility(false);
			} else if (fixesError == true) {
				jobAdapter.tooManyFixes();
				jobAdapter.notifyDataSetChanged();
				setProgressBarIndeterminateVisibility(false);
			}
		}
    };
	
    /**
	 * Gets the job information, including the jobSpec and the associated fixes, then calls the
	 * handler to update the view.
	 *
	 * @param jobId the job name (required)
	 * 
	 * @see com.perforce.p4scout.JobDetail#detailUpdateHandle
	 */
	private void getJob(final String jobId) {
		class Worker extends Thread {
		    @Override
			public void run() {
				try {		    
					serverCaller.connect(true);

					if (Thread.interrupted()) {
						return;
					}
					
				    jobData = serverCaller.getJob(jobId);
				    if (jobData.getDescription().trim().contentEquals("<enter description here>")) {
				    	throw new NoSuchObjectException(getString(R.string.strJob) + " " + jobId +
				    			" " + getString(R.string.strNonExist));
				    }
				    
					if (Thread.interrupted()) {
						return;
					}
					
					jobSpec = serverCaller.getJobSpec();
					if (Thread.interrupted()) {
						return;
					}
				    detailUpdateHandle.post(detailUpdateUpdater);
				    try {						
				    	jobFixes = serverCaller.getFixes(jobId);
				    } catch (Throwable t) {
				    	jobFixes = null;
				    	fixesError = true;
				    }

					if (Thread.interrupted()) {
						return;
					}
				    detailUpdateHandle.post(detailUpdateUpdater);
				    serverCaller.disconnect();
				    
				} catch (final Exception e) {
					if (Thread.interrupted()) {
						return;
					}
					errorHandle.post(new Runnable() {
						@Override
						public void run() {
							Toast.makeText(getApplicationContext(), e.getMessage().trim(),
									Toast.LENGTH_LONG).show();
							if (e.getMessage().trim().contains(PASSWORDINVALID) || 
									e.getMessage().trim().contains(SESSIONLOGGEDOUT) ||
					    			e.getMessage().trim().contains(PASSWORDNOTSET)) {
								Intent loginIntent = new Intent(context, LoginDialog.class);
								loginIntent.putExtra(ID, serverId);
								loginIntent.putExtra(ERROR, e.getMessage());
								loginIntent.putExtra(CALLER, getLocalClassName());
					    		startActivity(loginIntent);
							}
						}
					});
		    		    		
					e.printStackTrace();
					finish();
				}
		    }
		}
		runner[0] = new Worker();
		runner[0].start();
	}

	/**
	 * This is the initialisation method for the job detail view.<br/>
	 * This displays the view and kicks off the Perforce commands in a new thread.
	 * 
	 * @see android.app.Activity#onCreate(android.os.Bundle)
	 */
	@Override
    protected void onCreate(Bundle savedInstanceState) {
		super.onCreate(savedInstanceState);
		
		requestWindowFeature(Window.FEATURE_INDETERMINATE_PROGRESS);
		setContentView(R.layout.jobdetail);
		setProgressBarIndeterminateVisibility(true);
				
		Intent i = getIntent();
		Bundle extras = i.getExtras();
		
		context = this;
		errorHandle = new Handler(Looper.getMainLooper());
		
		String jobId = (String) extras.get(JOBID);
		serverId = (String) extras.get(ID);
		
		runner = new Thread[1];
		
		loading = new ProgressDialog(this);
		loading.setIndeterminate(true);
		loading.setMessage(getString(R.string.tvLoading));
		loading.show();
		
		serverCaller = new PerforceSession(this, serverId);
			
		setTitle(getString(R.string.strJobSpc) + " " + jobId + " - " + serverCaller.getServerName());
		
		jobAdapter = new JobDetailAdapter(context);
		ListView jobDetailsList = (ListView) findViewById(R.id.lvJobDetails);
		jobDetailsList.setAdapter(jobAdapter);
		
		jobDetailsList.setOnItemClickListener(new OnItemClickListener() {
			@Override
			public void onItemClick(AdapterView<?> parent, View view, int position, long id) {
				if (jobAdapter.getItem(position) == ItemType.FIX &&
						jobAdapter.getFix(position) != null) {
					Intent i = new Intent(view.getContext(), ChangeDetail.class);
					i.putExtra(CHANGENUM, jobAdapter.getFix(position).getChangelistId());
					i.putExtra(ID, serverId);
					startActivity(i);
				}
			}
		});
		
		getJob(jobId);
		
	}
	
	/**
     * This kills all background threads when we exit. There is no need for them to consume
     * resources when we are doing something else.
     */
	@Override
    protected void onPause() {
    	super.onPause();
    	if (runner == null) {
    		return;
    	}
    	for (int i=0; i<runner.length; i++) {
    		if (runner[i] != null) {
    			runner[i].interrupt();
    		}
    	}
    }
	
	/**
	 * This prevents the activity from being restarted when the orientation changes
	 */
    @Override
    public void onConfigurationChanged(Configuration newConfig){        
        super.onConfigurationChanged(newConfig);
    }
}
