/**
 * Copyright (c) 2010 Perforce Software.  All rights reserved.
 */
package com.perforce.p4scout;

import static com.perforce.p4scout.Constants.CALLER;
import static com.perforce.p4scout.Constants.ERROR;
import static com.perforce.p4scout.Constants.ID;
import static com.perforce.p4scout.Constants.JOBID;
import static com.perforce.p4scout.Constants.PASSWORDINVALID;
import static com.perforce.p4scout.Constants.PASSWORDNOTSET;
import static com.perforce.p4scout.Constants.SESSIONLOGGEDOUT;

import java.util.List;

import android.app.Activity;
import android.app.ProgressDialog;
import android.content.Context;
import android.content.Intent;
import android.content.SharedPreferences;
import android.content.res.Configuration;
import android.os.Bundle;
import android.os.Handler;
import android.os.Looper;
import android.preference.PreferenceManager;
import android.view.View;
import android.view.View.OnClickListener;
import android.widget.AdapterView;
import android.widget.EditText;
import android.widget.ImageButton;
import android.widget.ListView;
import android.widget.Toast;
import android.widget.AdapterView.OnItemClickListener;

import com.perforce.p4java.core.IJob;

/**
 * This class provides a way to search Perforce jobs and display the result as a list.<br/>
 * These are Perforce Server details are passed as an extra:<p>
 * 
 * <ul><li>string ID The ServerId from the P4SERVERS table (required)</ul>
 * 
 */
public class JobList  extends Activity {

	protected String serverId;
	protected PerforceSession serverCaller;
	protected JobListAdapter jobListAdapter;
	private Context context;
	private ProgressDialog loading;
	private int jobsToGet;
	private Thread[] runner;

	/** The Handler for updating the job details in the View */
	private Handler errorHandle;
	
	/** The Handler for updating the list of jobs in the View */
	private Handler jobSearchHandle = new Handler();
	
	/**
     * The Runnable for updating the list of jobs in the View.
     * 
     * @see com.perforce.p4scout.JobList#doSearch(String)
     */
    private Runnable jobSearchUpdater = new Runnable() 
	{
		public void run()
		{
			jobListAdapter.notifyDataSetChanged();
			loading.dismiss();
		}
		
	};
	
	/**
	 * Do the job search, and call the update handler when done.
	 *
	 * @param search the Perforce job search string. (required)
	 * 
	 * @see com.perforce.p4scout.JobList#jobSearchHandle
	 */
	private void doSearch(final String search) {
		class Worker extends Thread {
		    @Override
			public void run() {
		    	try {
		    		serverCaller.connect(true);
		    		List<IJob> jobList = serverCaller.searchJobs(search, jobsToGet);
		    		if (Thread.interrupted()) {
						return;
					}
		        	for (int j=0; j<jobList.size(); j++) {
		        		jobListAdapter.addJob(jobList.get(j));
		        	}
		    	    serverCaller.disconnect();		    	    
		    	} catch (final Exception e) {
		    		if (Thread.interrupted()) {
						return;
					}
					errorHandle.post(new Runnable() {
						@Override
						public void run() {
							Toast.makeText(getApplicationContext(), e.getMessage().trim(),
									Toast.LENGTH_LONG).show();
							if (e.getMessage().trim().contains(PASSWORDINVALID) || 
									e.getMessage().trim().contains(SESSIONLOGGEDOUT) ||
					    			e.getMessage().trim().contains(PASSWORDNOTSET)) {
								Intent loginIntent = new Intent(context, LoginDialog.class);
								loginIntent.putExtra(ID, serverId);
								loginIntent.putExtra(ERROR, e.getMessage());
								loginIntent.putExtra(CALLER, getLocalClassName());
					    		startActivity(loginIntent);
							}
						}
					});
		    		    		
					e.printStackTrace();
				}
				
		    	jobSearchHandle.post(jobSearchUpdater);
		    }
		}
		runner[0] = new Worker();
		runner[0].start();
    }
	
	/**
	 * This is the initialisation method for the Job search view.<br/>
	 * This displays the view and sets up the search plumbing.
	 * 
	 * @see android.app.Activity#onCreate(android.os.Bundle)
	 */
	@Override
	protected void onCreate(Bundle savedInstanceState)
	{
		super.onCreate(savedInstanceState);
		setContentView(R.layout.jobslist);
		
		context = this;
		errorHandle = new Handler(Looper.getMainLooper());
		runner = new Thread[1];
		
		Intent i = getIntent();
		Bundle extras = i.getExtras();
		serverId = (String) extras.get(ID);
		
		serverCaller = new PerforceSession(this, serverId);
	    setTitle(getString(R.string.strJobSearch) + " - " + serverCaller.getServerName());
		
		loading = new ProgressDialog( context );
		loading.setMessage(getString(R.string.tvLoading));
		loading.setIndeterminate(true);
		
		SharedPreferences sp = PreferenceManager.getDefaultSharedPreferences(this);
		jobsToGet = Integer.parseInt(sp.getString("prefGetJobs", "25"));
		
    	OnItemClickListener listClickListen = new OnItemClickListener() {
			public void onItemClick(AdapterView<?> parent, View view, int position, long id) {
				
				JobListAdapter jobListAdapter = (JobListAdapter) parent.getAdapter();
				Intent jobIntent = new Intent(context, JobDetail.class);
				jobIntent.putExtra(JOBID, jobListAdapter.getItem(position).getId());
				jobIntent.putExtra(ID, serverId);
				startActivity(jobIntent);
				
			}
    	};
    	
    	ListView jobsList = (ListView) findViewById(R.id.lvJobs);
    	jobListAdapter = new JobListAdapter(this);
    	jobsList.setAdapter(jobListAdapter);
    	jobsList.setOnItemClickListener(listClickListen);

		
    	OnClickListener buttonClickListen = new OnClickListener() {
			public void onClick(View view) {
				if (((EditText) findViewById(R.id.etJobSearchBox)).getText().toString().length() > 0) {
					jobListAdapter.clear();
					loading.show();
					doSearch(((EditText) findViewById(R.id.etJobSearchBox)).getText().toString());
				}
			}
			
    	};
    	
    	ImageButton jobsSearch = (ImageButton) findViewById(R.id.btJobSearch);
    	jobsSearch.setOnClickListener(buttonClickListen);
    	
	}
	

	/**
     * This kills all background threads when we exit. There is no need for them to consume
     * resources when we are doing something else.
     */
	@Override
    protected void onPause() {
    	super.onPause();
    	if (runner == null) {
    		return;
    	}
    	for (int i=0; i<runner.length; i++) {
    		if (runner[i] != null) {
    			runner[i].interrupt();
    		}
    	}
    }
	
	/**
	 * This prevents the activity from being restarted when the orientation changes
	 */
    @Override
    public void onConfigurationChanged(Configuration newConfig){        
        super.onConfigurationChanged(newConfig);
    }
}
