/**
 * Copyright (c) 2010 Perforce Software.  All rights reserved.
 */
package com.perforce.p4scout;

import static android.provider.BaseColumns._ID;
import static com.perforce.p4scout.Constants.ERROR;
import static com.perforce.p4scout.Constants.ID;
import static com.perforce.p4scout.Constants.P4PASS;
import static com.perforce.p4scout.Constants.P4SERVERS;
import static com.perforce.p4scout.Constants.P4USER;
import static com.perforce.p4scout.Constants.PASSWORDINVALID;
import android.app.Activity;
import android.app.ProgressDialog;
import android.content.ContentValues;
import android.content.Context;
import android.content.DialogInterface;
import android.content.Intent;
import android.content.SharedPreferences;
import android.content.DialogInterface.OnCancelListener;
import android.content.res.Configuration;
import android.database.sqlite.SQLiteDatabase;
import android.os.Bundle;
import android.preference.PreferenceManager;
import android.view.View;
import android.view.Window;
import android.view.View.OnClickListener;
import android.widget.Button;
import android.widget.CheckBox;
import android.widget.EditText;
import android.widget.Toast;

/**
 * This is the class that displays the login dialog.<br>
 * This will be displayed if the user needs to log in, but has not set a password to use.<p>
 * The ServerId extra, ID, is required.
 */
public class LoginDialog extends Activity {

	protected String serverId, errMsg, oldUser;
	protected PerforceSession serverCaller;
	
	/**
	 * This is the initialisation method for the login dialog.<br/>
	 * If a specific username is set and not invalid, we offer it as the default.
	 * 
	 * @see android.app.Activity#onCreate(android.os.Bundle)
	 */
	@Override
	protected void onCreate(Bundle savedInstanceState)
	{
		super.onCreate(savedInstanceState);
		
		Window w = getWindow();
		w.requestFeature(Window.FEATURE_LEFT_ICON);
		setContentView(R.layout.login);
		w.setFeatureDrawableResource(Window.FEATURE_LEFT_ICON, android.R.drawable.ic_dialog_alert);
		
		Intent i = getIntent();
		Bundle extras = i.getExtras();
		final Context context = this;
		
		errMsg = extras.getString(ERROR);
		serverId = extras.getString(ID);

		serverCaller = new PerforceSession(this, serverId);
	    oldUser = serverCaller.getServerUser();
	    
	    if (!serverCaller.canUseLogin()) {
	    	Toast tt = Toast.makeText(context, getString(R.string.strNoSDCard), Toast.LENGTH_LONG);
			tt.show();
			finish();
	    }
	    
		P4Data storedServers = new P4Data(this);
	    final SQLiteDatabase db = storedServers.getWritableDatabase();
				
		final EditText userNameBox = (EditText) findViewById(R.id.etServerUser);
		final EditText passwdBox = (EditText) findViewById(R.id.etServerPass);
		final ProgressDialog loading = new ProgressDialog(this);
		loading.setIndeterminate(true);
		loading.setOnCancelListener(new OnCancelListener() {
			@Override
			public void onCancel(DialogInterface arg0) {
				finish();
			}
		});
		loading.setMessage(getString(R.string.tvLoggingIn));
		
		final SharedPreferences sp = PreferenceManager.getDefaultSharedPreferences(this);
		
		if (errMsg != null && errMsg.trim().contentEquals(PASSWORDINVALID)) {
			userNameBox.setText(oldUser);
		} else {
			userNameBox.setHint(sp.getString("etpDefaultUser", "nouser"));			
		}
		
		((Button) findViewById(R.id.btOK)).setOnClickListener(new OnClickListener() {
			@Override
			public void onClick(View v) {
				try {
					loading.show();
				    String userName = userNameBox.getText().toString();
					serverCaller.setServerUser(userName);
					serverCaller.setServerPasswd(passwdBox.getText().toString().trim());
					serverCaller.connect(true, true);
					
				    serverCaller.disconnect();
				    
				    // Setup for db updates
				    int update = 0;
				    ContentValues values = new ContentValues();
				    
				    if (userName.trim().length() <= 0) {
				    	// No username was specified. Use the default instead.
				    	userName = sp.getString("etpDefaultUser", "nouser");
				    }
				    
				    if (!userName.trim().contentEquals(oldUser)) {
				    	// Need to save the new username
				    	values.put(P4USER, userNameBox.getText().toString().trim());
				    	update++;
				    }
				    
				    if (((CheckBox) findViewById(R.id.cbStorePw)).isChecked()) {
				    	// Need to save the new password
				    	values.put(P4PASS, passwdBox.getText().toString().trim());
				    	update++;
				    }
				    
				    if (update > 0) {
				    	db.update(P4SERVERS, values, _ID + "=?", new String[] {serverId});
				    }
				    
				    
				    Toast tt = Toast.makeText(context, getString(R.string.strLoggedIn),
				    		Toast.LENGTH_LONG);
				    tt.show();
				    
				    db.close();
				    loading.dismiss();
					finish();
					
				} catch (Exception e) {
					if (loading.isShowing()) {
					    loading.dismiss();
					}
					Toast tt = Toast.makeText(context, e.getMessage().trim(), Toast.LENGTH_LONG);
					tt.show();
					e.printStackTrace();
					finish();
				}
			}	
		});
		
		((Button) findViewById(R.id.btCancel)).setOnClickListener(new OnClickListener() {
			@Override
			public void onClick(View v) {
			    db.close();
				finish();
			}	
		});
	}
	
	/**
	 * This prevents the activity from being restarted when the orientation changes
	 */
    @Override
    public void onConfigurationChanged(Configuration newConfig){        
        super.onConfigurationChanged(newConfig);
    }
}
