/**
 * Copyright (c) 2010 Perforce Software.  All rights reserved.
 */
package com.perforce.p4scout;

import static android.provider.BaseColumns._ID;
import static com.perforce.p4scout.Constants.DESC;
import static com.perforce.p4scout.Constants.NAME;
import static com.perforce.p4scout.Constants.NOLOGINREQUIRED;
import static com.perforce.p4scout.Constants.NOMONITOR;
import static com.perforce.p4scout.Constants.P4PASS;
import static com.perforce.p4scout.Constants.P4PORT;
import static com.perforce.p4scout.Constants.P4SERVERS;
import static com.perforce.p4scout.Constants.P4USER;
import static com.perforce.p4scout.Constants.PASSWORDINVALID;
import static com.perforce.p4scout.Constants.PASSWORDNOTSET;
import static com.perforce.p4scout.Constants.SESSIONLOGGEDOUT;
import static com.perforce.p4scout.Constants.TICKETVALID;

import java.net.URISyntaxException;
import java.util.Date;
import java.util.LinkedList;
import java.util.List;
import java.util.Properties;

import android.app.Activity;
import android.content.Context;
import android.content.SharedPreferences;
import android.content.pm.PackageInfo;
import android.database.Cursor;
import android.database.sqlite.SQLiteDatabase;
import android.os.Environment;
import android.preference.PreferenceManager;

import com.perforce.p4java.PropertyDefs;
import com.perforce.p4java.core.IChangelist;
import com.perforce.p4java.core.IChangelist.Type;
import com.perforce.p4java.core.IChangelistSummary;
import com.perforce.p4java.core.IFix;
import com.perforce.p4java.core.IJob;
import com.perforce.p4java.core.IJobSpec;
import com.perforce.p4java.core.IServerProcess;
import com.perforce.p4java.core.IUser;
import com.perforce.p4java.core.IUserSummary;
import com.perforce.p4java.core.file.FileSpecBuilder;
import com.perforce.p4java.core.file.FileSpecOpStatus;
import com.perforce.p4java.core.file.IFileSpec;
import com.perforce.p4java.exception.P4JavaException;
import com.perforce.p4java.exception.RequestException;
import com.perforce.p4java.option.server.GetChangelistsOptions;
import com.perforce.p4java.option.server.GetDepotFilesOptions;
import com.perforce.p4java.option.server.GetFixesOptions;
import com.perforce.p4java.option.server.GetJobsOptions;
import com.perforce.p4java.server.IOptionsServer;
import com.perforce.p4java.server.IServerInfo;
import com.perforce.p4java.server.ServerFactory;
import com.perforce.p4scout.ServerInfo.Status;
import com.perforce.p4scout.ServerInfo.Warning;

/**
 * Perforce Session class is a helper class to reduce repetitive code.<p>
 * We grab most of our connection infomation from the database, so we include that code here too.
 */
public class PerforceSession {

	private IOptionsServer server;
	private String serverId, serverName, serverDesc, serverAddr, serverUser, serverPasswd;
	private int working;
	private Activity context;
	private boolean canLogin;
	
	public static int PerforceTest(Context locContext, String serverUri, String user, String pass)
			throws P4JavaException, URISyntaxException {
		
		if (user != null && user.length() <= 0) {
			user = null;
		}		
		if (pass != null && pass.length() <= 0) {
			pass = null;
		}
		
		int returnVal = 0;
		Properties props = new Properties();
		if (!Environment.getExternalStorageState().equals(Environment.MEDIA_MOUNTED)) {
			pass = null;
		} else {
			String sdCard = Environment.getExternalStorageDirectory().getPath();
			props.setProperty(PropertyDefs.TICKET_PATH_KEY, sdCard + "/p4tickets.txt");
		}
		
		try {
			// Get the version details for this package from the manifest file
			PackageInfo apkInfo = locContext.getPackageManager().getPackageInfo(
					locContext.getPackageName(), 0);
			props.setProperty(PropertyDefs.PROG_NAME_KEY,
					locContext.getString(R.string.tvVersionShort));
			props.setProperty(PropertyDefs.PROG_VERSION_KEY,
					apkInfo.versionName + "/" + String.valueOf(apkInfo.versionCode));
		} catch (Exception e) {
			// Ignore them...
		}
		String addr = "p4java://" + serverUri;
		if (user != null) {
			addr += "?userName=" + user;
		}
		
		IOptionsServer tmpServer = ServerFactory.getOptionsServer(addr, props);
		tmpServer.connect();
		
		if (user != null) {
			// A user is set. Can we authenticate?
			tmpServer.logout();
			String isLoggedIn = tmpServer.getLoginStatus();
			if (isLoggedIn != null && !isLoggedIn.contentEquals(TICKETVALID)
	    		&& !isLoggedIn.contentEquals(NOLOGINREQUIRED)) {
				returnVal = 1;
		    	if (pass.length() > 0) {
		    		tmpServer.login(pass);
		    		returnVal = 0;
		    	}
			}
		}
		
		tmpServer.disconnect();
		return returnVal;
	}
	
	/**
	 * Instantiates a new Perforce session.
	 *
	 * @param context the parent Activity. (required)
	 * @param serverId the server id in the database. (required)
	 */
	public PerforceSession(Activity context, String serverId) {
		this.context = context;
		this.serverId = serverId;
		working = 0;
		
		if (Environment.getExternalStorageState().equals(Environment.MEDIA_MOUNTED)) {
			canLogin = true;
		} else {
			canLogin = false;
		}
		
		P4Data storedServers = new P4Data(context);
    	SQLiteDatabase db = storedServers.getReadableDatabase();
    	Cursor cursor = db.query(P4SERVERS, new String[] {P4PORT, NAME, DESC, P4USER, P4PASS},
    			_ID + "=?", new String[] {serverId}, null, null, null);
	    context.startManagingCursor(cursor);
	    int serverName = cursor.getColumnIndexOrThrow(NAME);
	    int serverDesc = cursor.getColumnIndexOrThrow(DESC);
	    int serverUser = cursor.getColumnIndexOrThrow(P4USER);
	    int serverPasswd = cursor.getColumnIndexOrThrow(P4PASS);
	    int serverAddr = cursor.getColumnIndexOrThrow(P4PORT);
	    cursor.moveToNext();
	    this.serverName = cursor.getString(serverName);
		this.serverDesc = cursor.getString(serverDesc);
		this.serverAddr = cursor.getString(serverAddr);
		this.serverUser = cursor.getString(serverUser);
		this.serverPasswd = cursor.getString(serverPasswd);
	    cursor.close();
	    context.stopManagingCursor(cursor);
	    db.close();
	}
	
	/**
	 * Connect to the Perforce Server, and authenticate if required.
	 *
	 * @param useLogin do the login. (required)
	 * @throws RequestException Perforce exceptions that we explicitly handle
	 * @throws P4JavaException any other Perforce related exception
	 * @throws URISyntaxException exception when parsing the URI string
	 */
	public void connect(boolean useLogin) throws RequestException, P4JavaException, 
			URISyntaxException {
		connect(useLogin, false);
	}
	
	/**
	 * Connect to the Perforce Server, and authenticate if required.
	 *
	 * @param useLogin do the login. (required)
	 * @param force forces logins to happen. (required)
	 * @throws RequestException Perforce exceptions that we explicitly handle
	 * @throws P4JavaException any other Perforce related exception
	 * @throws URISyntaxException exception when parsing the URI string
	 */
	public void connect(boolean useLogin, boolean force) throws RequestException, P4JavaException, 
			URISyntaxException {
		
		Properties props = new Properties();
		if (Environment.getExternalStorageState().equals(Environment.MEDIA_MOUNTED)) {
			canLogin = true;
			String sdCard = Environment.getExternalStorageDirectory().getPath();
			props.setProperty(PropertyDefs.TICKET_PATH_KEY, sdCard + "/p4tickets.txt");
		} else {
			canLogin = false;
			// Refresh password.
		}
		
		try {
			// Get the version details for this package from the manifest file
			PackageInfo apkInfo = context.getPackageManager().getPackageInfo(
					context.getPackageName(), 0);
			props.setProperty(PropertyDefs.PROG_NAME_KEY,
					context.getString(R.string.tvVersionShort));
			props.setProperty(PropertyDefs.PROG_VERSION_KEY,
					apkInfo.versionName + "/" + String.valueOf(apkInfo.versionCode));
		} catch (Exception e) {
			// Ignore them...
		}
		String addr = "p4java://" + serverAddr.trim() + "?userName=" + serverUser.trim();
		
		if (server == null || useLogin) {
			server = ServerFactory.getOptionsServer(addr, props);
		}
		
		if (!server.isConnected()) {
			server.connect();
		}
		
		if (server.supportsUnicode()) {
	    	server.setCharsetName("utf8");
	    }
		
		if (useLogin && force && canLogin) {
			try {
				server.logout();
			} catch (Throwable e) {
				// We probably wern't logged in!
			}
    		server.login(serverPasswd);
		} else if (useLogin && canLogin) {
			String isLoggedIn = server.getLoginStatus();
		    if (isLoggedIn != null && !isLoggedIn.contentEquals(TICKETVALID)
		    		&& !isLoggedIn.contentEquals(NOLOGINREQUIRED)) {
		    	//We need to be logged in
		    	if (serverPasswd.length() > 0) {
		    		server.login(serverPasswd);
		    	}
		    }
		}
	}

	/**
	 * Disconnect from the Perforce Server.
	 *
	 * @throws P4JavaException any Perforce related exception
	 */
	public void disconnect() throws P4JavaException {
		if (working <= 0 && server.isConnected()) {
			server.disconnect();
		}
	}
	
	/**
	 * Check that there are not too many files in a changelist.
	 *
	 * @throws RequestException Perforce exceptions that we explicitly handle
	 * @throws P4JavaException any Perforce related exception
	 */
	public List<IFileSpec> changeExceedsLimit(int changeId, int limit) throws RequestException,
			P4JavaException {
		// If ther Perforce Server is at least 2009.2, we can use the undoc "p4 files -m" flag.
		List<IFileSpec> items = server.getDepotFiles(FileSpecBuilder.makeFileSpecList("//...@" +
				String.valueOf(changeId) + ",@" + String.valueOf(changeId)), 
				new GetDepotFilesOptions().setMaxResults(limit+1));
		if (items.size() > 0) {
			for (IFileSpec item : items) {
				if (item.getOpStatus() != null && item.getOpStatus().equals(FileSpecOpStatus.ERROR)) {
					throw new P4JavaException();
				}
			}
		}
    	if (items.size() >= limit+1) {
    		return null;
    	}
    	return items;
	}
	
	
	/**
	 * Gets a changelist.
	 *
	 * @param changeId the changelist number. (required)
	 * @return the changelist object
	 * @throws RequestException Perforce exceptions that we explicitly handle
	 * @throws P4JavaException any other Perforce related exception
	 * @throws URISyntaxException exception when parsing the URI string
	 */
	public IChangelist getChangelist(int changeId) throws RequestException, P4JavaException,
			URISyntaxException {
		if (server == null || !server.isConnected()) {
			connect(true);
		}
		working++;
		IChangelist change = server.getChangelist(changeId);
		working--;
		return change;
	}
	
	/**
	 * Gets the X most recent changelists.
	 *
	 * @param changesToGet the number of changes to get
	 * @return a non-null, but possibly empty List of changelists
	 * @throws RequestException Perforce exceptions that we explicitly handle
	 * @throws P4JavaException any other Perforce related exception
	 * @throws URISyntaxException exception when parsing the URI string
	 */
	public List<IChangelistSummary> getChangelists(int changesToGet) throws RequestException,
			P4JavaException, URISyntaxException {
		if (server == null || !server.isConnected()) {
			connect(true);
		}
		working++;
		List<IChangelistSummary> changes = server.getChangelists(null, new GetChangelistsOptions()
				.setMaxMostRecent(changesToGet).setType(Type.SUBMITTED).setLongDesc(true));
		working--;
		return changes;
	}
	
	/**
	 * Gets the fixes related to a job.
	 *
	 * @param jobId the name of the job to get the fixes for. (required)
	 * @return the a List of fixes.
	 * @throws RequestException Perforce exceptions that we explicitly handle
	 * @throws P4JavaException any other Perforce related exception
	 * @throws URISyntaxException exception when parsing the URI string
	 */
	public List<IFix> getFixes(String jobId) throws RequestException, P4JavaException,
			URISyntaxException {
		if (server == null || !server.isConnected()) {
			connect(true);
		}
		working++;
		List<IFix> jobFixes = server.getFixes(null, new GetFixesOptions().setJobId(jobId));
		working--;
		return jobFixes;
	}
	
	/**
	 * Gets the job.
	 *
	 * @param jobId the name of the job to fetch. (required)
	 * @return the job object
	 * @throws RequestException Perforce exceptions that we explicitly handle
	 * @throws P4JavaException any other Perforce related exception
	 * @throws URISyntaxException exception when parsing the URI string
	 */
	public IJob getJob(String jobId) throws RequestException, P4JavaException, URISyntaxException {
		if (server == null || !server.isConnected()) {
			connect(true);
		}
		working++;
		IJob job = server.getJob(jobId);
		working--;
		return job;
	}
	
	/**
	 * Gets the job spec for this Perforce Server.
	 *
	 * @return the job spec object
	 * @throws RequestException Perforce exceptions that we explicitly handle
	 * @throws P4JavaException any other Perforce related exception
	 * @throws URISyntaxException exception when parsing the URI string
	 */
	public IJobSpec getJobSpec() throws RequestException, P4JavaException, URISyntaxException {
		if (server == null || !server.isConnected()) {
			connect(true);
		}
		working++;
		IJobSpec jobSpec = server.getJobSpec();
		working--;
		return jobSpec;
	}
	
	/**
	 * Gets the output of "p4 monitor show" to find long running commands.
	 *
	 * @return a List of Perforce processes
	 * @throws RequestException Perforce exceptions that we explicitly handle
	 * @throws P4JavaException any other Perforce related exception
	 * @throws URISyntaxException exception when parsing the URI string
	 */
	public List<IServerProcess> getMonitor() throws RequestException, P4JavaException,
			URISyntaxException {
		// Ensure we are logged in!
		connect(true);
		working++;
		List<IServerProcess> procs = server.getServerProcesses();
		working--;
		return procs;
	}
	
	/**
	 * Gets the server address.
	 *
	 * @return the server address (P4PORT)
	 */
	public String getServerAddr() {
		return serverAddr;
	}
	
	/**
	 * Gets the server description.
	 *
	 * @return the server description.
	 */
	public String getServerDesc() {
		return serverDesc;
	}
	
	/**
	 * Gets the server ID.
	 *
	 * @return the server ID
	 */
	public String getServerId() {
		return serverId;
	}
	
	/**
	 * Gets the "p4 info".
	 *
	 * @return the server info object
	 * @throws RequestException Perforce exceptions that we explicitly handle
	 * @throws P4JavaException any other Perforce related exception
	 * @throws URISyntaxException exception when parsing the URI string
	 */
	public IServerInfo getServerInfo() throws RequestException, P4JavaException,
			URISyntaxException {
		if (server == null || !server.isConnected()) {
			connect(true);
		}
		working++;
		IServerInfo info = server.getServerInfo();
		working--;
		return info;
	}

	/**
	 * Gets the server name.
	 *
	 * @return the server name
	 */
	public String getServerName() {
		return serverName;
	}
	
	/**
	 * Gets the IServer object.
	 *
	 * @return the server object
	 */
	public IOptionsServer getServerObject() {
		return server;
	}

	/**
	 * Gets the Perforce user's username.
	 *
	 * @return the username
	 */
	public String getServerUser() {
		return serverUser;
	}

	/**
	 * Gets the user information for a specific user.
	 *
	 * @param userName the Perforce user's username. (required)
	 * @return the user object
	 * @throws RequestException Perforce exceptions that we explicitly handle
	 * @throws P4JavaException any other Perforce related exception
	 * @throws URISyntaxException exception when parsing the URI string
	 */
	public IUser getUser(String userName) throws RequestException, P4JavaException,
			URISyntaxException {
		if (server == null || !server.isConnected()) {
			connect(true);
		}
		working++;
		IUser user = server.getUser(userName);
		working--;
		return user;
	}

	/**
	 * Gets a list of all of the users on the Perforce Server.
	 *
	 * @return a List of user summary objects
	 * @throws RequestException Perforce exceptions that we explicitly handle
	 * @throws P4JavaException any other Perforce related exception
	 * @throws URISyntaxException exception when parsing the URI string
	 */
	public List<IUserSummary> getUsers() throws RequestException, P4JavaException,
			URISyntaxException {
		if (server == null || !server.isConnected()) {
			connect(false);
		}
		working++;
		List<IUserSummary> users = server.getUsers(null, null);
		working--;
		return users;
	}
	
	/**
	 * Gets any warning for the Perforce Server.
	 *
	 * @param serverInfo the server info object. (required)
	 * @return the warning status
	 * @throws RequestException Perforce exceptions that we explicitly handle
	 * @throws P4JavaException any other Perforce related exception
	 * @throws URISyntaxException exception when parsing the URI string
	 */
	public Status getWarnings(ServerInfo serverInfo) throws RequestException, P4JavaException,
			URISyntaxException {
		if (server == null || !server.isConnected()) {
			connect(false);
		}
		working++;
	    
	    String[] license = server.getServerInfo().getServerLicense().split(" ");
	    int u = license.length - 2;
	    
	    int maxUsers = 0;
	    int expiresIn = 1000;
	    
	    while (u >= 0 && !license[u+1].equals("users")) {
	    	u--;
	    }
	    if (u >= 0) {
	    	// There is a license file in place
	    	maxUsers = Integer.parseInt(license[u]);
	    	String licenseDays = license[license.length - 1].substring(0,
	    			license[license.length - 1].length()-1);
	    	
	    	long expiry = Date.parse(licenseDays);
		    
		    long expires = expiry - new Date().getTime();
		    if (expires > 0)
		    {
		    	expiresIn = (int) (expires / (1000 * 60 * 60 * 24));
		    }
		    else
		    {
		    	expiresIn = -1;
		    }
		    
	    } else {
	    	// No license...
	    	maxUsers = 2;
	    	expiresIn = -2;
	    }
	    
	    SharedPreferences sp = PreferenceManager.getDefaultSharedPreferences(context);
		Status returnStatus = Status.ONLINE;
		
	    // Do license expiry check
		serverInfo.setLicensesExpiresIn(expiresIn);
	    if (expiresIn > -2 &&
	    		expiresIn < Integer.parseInt(sp.getString("prefExpiryThreshold", "30"))) {
    		serverInfo.setWarning(Warning.LICENSE);
	    	returnStatus = Status.WARNINGS;
	    }
	    
	    // Get full list of users for us to count
	    List<IUserSummary> users = getUsers();
	    serverInfo.setLicenses(users.size(), maxUsers);
	    if ((maxUsers - users.size()) < Integer.parseInt(sp.getString("prefUserThreshold", "5"))) {
    		serverInfo.setWarning(Warning.USERS);
	    	returnStatus = Status.WARNINGS;
	    }
	    
	    // If user doesn't exist, we don't want to create it
	    if (!userExists()) {
	    	working--;
		    server.disconnect();
		    return returnStatus;
	    }

	    //Do we have monitoring? 
    	try {
    		connect(true);
    		List<IServerProcess> procs = server.getServerProcesses();
    		
    		// If we got here, then yes
    		int longCount = 0;
		    int monThresh = Integer.parseInt(sp.getString("prefMonitorThreshold", "120"));
	    	for (IServerProcess proc: procs) {
	    		String t[] = proc.getTime().split(":");
				if ( ( (Integer.parseInt(t[0]) * 60 * 60) + (Integer.parseInt(t[1]) * 60) +
						Integer.parseInt(t[2]) ) > monThresh ) {
					// Running time in seconds greater than threshold
					longCount++;
				}
	    	}
	    	if (longCount > 0) {
	    		serverInfo.setLongCmds(longCount);
	    		serverInfo.setWarning(Warning.MONITOR);
				returnStatus = Status.WARNINGS;
	    	}
	    	
    	} catch (RequestException re) {
    		if (!re.getMessage().trim().contentEquals(NOMONITOR)
    				&& !re.getMessage().trim().contentEquals(PASSWORDINVALID)
	    			&& !re.getMessage().trim().contains(PASSWORDNOTSET)
    				&& !re.getMessage().trim().contains(SESSIONLOGGEDOUT)) {
    			// Possibly, but something else happened here
    			throw re;
    		}
    		// Monitor is not enabled
    		// Lets continue
    	}
    	working--;
    	return returnStatus;
	}

	/**
	 * Search the jobs.
	 *
	 * @param searchStr the string to search the jobs for. (required)
	 * @param jobsToGet 
	 * @return a List of job objects
	 * @throws RequestException Perforce exceptions that we explicitly handle
	 * @throws P4JavaException any other Perforce related exception
	 * @throws URISyntaxException exception when parsing the URI string
	 */
	public List<IJob> searchJobs(String searchStr, int jobsToGet) throws RequestException, 
			P4JavaException, URISyntaxException {
		if (server == null || !server.isConnected()) {
			connect(true);
		}
		working++;
		List<IJob> jobList = server.getJobs(null, new GetJobsOptions().setMaxJobs(jobsToGet)
				.setJobView(searchStr));
		working--;
		return jobList;
	}
	
	/**
	 * Sets the server password.
	 *
	 * @param userPasswd the new password. (required)
	 */
	public void setServerPasswd(String userPasswd) {
		serverPasswd = userPasswd;
	}
	
	/**
	 * Sets the Perforce user to use.
	 *
	 * @param userName the new username. (required)
	 */
	public void setServerUser(String userName) {
		serverUser = userName;
	}
	
	/**
	 * This method returns a check on whether we can access the device's SD card.
	 * If not, we cannot read/store tickets.
	 * 
	 * @return true if we can login to the Perforce Server
	 */
	public boolean canUseLogin() {
		return canLogin;
	}
	
	/**
	 * Does the current user exist?
	 *
	 * @return true, if the user exists
	 * @throws RequestException Perforce exceptions that we explicitly handle
	 * @throws P4JavaException any other Perforce related exception
	 * @throws URISyntaxException exception when parsing the URI string
	 */
	public boolean userExists() throws RequestException, P4JavaException, URISyntaxException {
		if (server == null || !server.isConnected()) {
			connect(false);
		}
		working++;
		List<String> userList = new LinkedList<String>();
	    userList.add(serverUser);
	    List<IUserSummary> users = server.getUsers(userList, null);
	    working--;
	    return users.size() > 0;
	}
}
