/**
 * Copyright (c) 2010 Perforce Software.  All rights reserved.
 */
package com.perforce.p4scout;

import static com.perforce.p4scout.Constants.ACTION;
import static com.perforce.p4scout.Constants.CALLER;
import static com.perforce.p4scout.Constants.EDIT;
import static com.perforce.p4scout.Constants.ERROR;
import static com.perforce.p4scout.Constants.ID;
import static com.perforce.p4scout.Constants.NOMONITOR;
import static com.perforce.p4scout.Constants.PASSWORDINVALID;
import static com.perforce.p4scout.Constants.PASSWORDNOTSET;
import static com.perforce.p4scout.Constants.SESSIONLOGGEDOUT;
import static com.perforce.p4scout.Constants.USERS;

import java.util.Date;
import java.util.LinkedList;
import java.util.List;
import java.util.concurrent.TimeUnit;
import java.util.concurrent.locks.Lock;
import java.util.concurrent.locks.ReentrantLock;

import android.app.Activity;
import android.app.ProgressDialog;
import android.content.Context;
import android.content.DialogInterface;
import android.content.Intent;
import android.content.SharedPreferences;
import android.content.DialogInterface.OnCancelListener;
import android.content.res.Configuration;
import android.os.Bundle;
import android.os.Handler;
import android.os.Looper;
import android.preference.PreferenceManager;
import android.view.Menu;
import android.view.MenuInflater;
import android.view.MenuItem;
import android.view.View;
import android.view.View.OnClickListener;
import android.widget.Button;
import android.widget.LinearLayout;
import android.widget.ProgressBar;
import android.widget.ScrollView;
import android.widget.TextView;
import android.widget.Toast;
import android.widget.TableLayout.LayoutParams;

import com.perforce.p4java.core.IServerProcess;
import com.perforce.p4java.core.IUserSummary;
import com.perforce.p4java.server.IServerInfo;

/**
 * This class displays informtion about a Perforce Server.<p>
 * 
 * Extras:
 * 
 * <ul><li>ID The ServerId from the P4SERVERS table (required)
 * <li>int USERS The number of users on the Perforce Server</ul>
 */
public class ServerDetail extends Activity {
    
	protected long expiry = 0;
	protected String serverId;
	protected List<IServerProcess> monitorCmds;
	protected ProgressBar fillBar;
	protected TextView userLicenses;
	protected LinearLayout monitorCmdList;
	protected int monitorCmdsNum;
	protected int preUserCount;
	protected int userCount = 0;
	protected int maxUsers = 0;
	protected boolean userWorking = true;
	protected boolean monitorWorking = true;
	protected Context context;
	private ProgressDialog loading;
	private PerforceSession serverCaller;
	private IServerInfo info;
	private SharedPreferences sp;
	private Thread[] runner;
	private final Lock warnStrLock = new ReentrantLock();

	/** The Handler for updating the job details in the View */
	private Handler errorHandle;
	
	/** The Handler for updating the Perforce Server details in the View */
	private Handler infoUpdateHandle = new Handler();
    
	/**
     * The Runnable for updating the Perforce Server details in the View.
     * This requires info to be populated.
     * 
     * @see com.perforce.p4scout.ServerDetail#getInfo(String, String)
     */
    private Runnable infoUpdateUpdater = new Runnable() 
	{
		public void run()
		{

			((ScrollView) findViewById(R.id.svServerView))
				.setVisibility(View.VISIBLE);
			((TextView) findViewById(R.id.tvTrueAddress))
				.setText(info.getServerAddress());
		    if (info.getServerUptime() != null) {
				((TextView) findViewById(R.id.tvUpTimeDays))
					.setText(new Integer (Integer.parseInt(
							info.getServerUptime().split(":")[0]) / 24).toString());
		    } else {
		    	findViewById(R.id.alServerUptime).setVisibility(View.INVISIBLE);
		    }
		    String[] verString = info.getServerVersion().split("/");
		    ((TextView) findViewById(R.id.tvPlatform))
				.setText("P4D/" + verString[1]);
		    ((TextView) findViewById(R.id.tvVersion))
				.setText(verString[2] + "/" + verString[3].split("\\(")[0]);
		    
		    String licenseDays;
		    String[] license = info.getServerLicense().split(" ");
		    int u = license.length - 2;
		    
		    while (u >= 0 && !license[u+1].equals("users")) {
		    	u--;
		    }
		    if (u >= 0) {
		    	// There is a license file in place
		    	maxUsers = Integer.parseInt(license[u]);
		    	licenseDays = license[license.length - 1].substring(0,
		    			license[license.length - 1].length()-1);
		    	
		    	expiry = Date.parse(licenseDays);
			    
			    long expires = expiry - new Date().getTime();
			    long expiresIn = 1000;
			    if (expires > 0) {
			    	expiresIn = expires / (1000 * 60 * 60 * 24);
			    	licenseDays = Long.toString(expires / (1000 * 60 * 60 * 24));
			    } else {
			    	expiresIn = -1;
			    	licenseDays = getString(R.string.strLicenseExpired);
			    }
			    
				if (expiresIn >= -1 &&
						expiresIn < Integer.parseInt(sp.getString("prefExpiryThreshold", "30"))) {
					((TextView) findViewById(R.id.tvWarnings))
						.setVisibility(View.VISIBLE);
					try {
						TextView warns = ((TextView) findViewById(R.id.tvWarningsText));
						warns.setVisibility(View.VISIBLE);
						warnStrLock.tryLock(120, TimeUnit.SECONDS);
						String warnsStr = warns.getText().toString();
						if (warnsStr.length() > 0) {
							warnsStr += "\n";
						}
						if (expiresIn >= 0) {
							// The license hasn't expired yet
							warns.setText(warnsStr + String.valueOf(expiresIn) + " " +
									getString(R.string.strWarnLicense));
						} else {
							// The license has already expired
							warns.setText(warnsStr + getString(R.string.strLicenseExpired));
						}
						warnStrLock.unlock();
					} catch (InterruptedException e) {
						loading.dismiss();
						return;
					}
					
				}
		    } else {
		    	// No license...
		    	maxUsers = 2;
		    	licenseDays = getString(R.string.strNoLicenseTimeout);
		    }

			userLicenses.setText(userCount + "/" + maxUsers);
			((TextView) findViewById(R.id.tvLicensesTime))
				.setText(licenseDays + " " + getString(R.string.tvLicenseDaysRemaining));
			
			fillBar = (ProgressBar) findViewById(R.id.pbUserCount);
			fillBar.setMax(maxUsers);
			
			// The rest we only ever do first time round
			if (loading.isShowing()) {
				fillBar.setIndeterminate(true);
				
				// Fetch users, but only if we havn't got them already
				if (preUserCount == -1) {
					getUsers();
				} else {
					userCount = preUserCount;
					userWorking = false;
					userCountUpdater.run();
				}
				getMonitor();
				loading.dismiss();
			}
		}
		
	};
	
	/** The Handler for updating the users information in the View */
	private Handler userCountHandle = new Handler();
    
	/**
     * The Runnable for updating the users information in the View.
     * This requires userCount to be set.
     * 
     * @see com.perforce.p4scout.ServerDetail#getUsers()
     */
    private Runnable userCountUpdater = new Runnable() 
	{
		public void run()
		{
			fillBar.setIndeterminate(false);
			if (userWorking) {
				fillBar.setSecondaryProgress(userCount);
			} else {
				fillBar.setSecondaryProgress(0);
				fillBar.setProgress(userCount);
				userLicenses.setText(userCount + "/" + maxUsers);
			}
			if ((maxUsers - userCount) < Integer.parseInt(sp.getString("prefUserThreshold", "5"))) {
				((TextView) findViewById(R.id.tvWarnings))
					.setVisibility(View.VISIBLE);
				try {
					TextView warns = ((TextView) findViewById(R.id.tvWarningsText));
					warns.setVisibility(View.VISIBLE);
					warnStrLock.tryLock(120, TimeUnit.SECONDS);
					String warnsStr = warns.getText().toString();
					if (warnsStr.length() > 0) {
						warnsStr += "\n";
					}
					warns.setText(warnsStr + String.valueOf(maxUsers - userCount) + " " +
							context.getString(R.string.strWarnUsers));
				} catch (InterruptedException e) {
					return;
				}
			}
		}
		
	};
	
	/** The Handler for updating the monitor information in the View */
	private Handler monitorHandle = new Handler();
    
	/**
     * The Runnable for updating the monitor information in the View.
     * This requires monitorCmds to be set.
     * 
     * @see com.perforce.p4scout.ServerDetail#getMonitor()
     */
    private Runnable monitorUpdater = new Runnable() 
	{
		public void run()
		{
			// Hide existing stuff first
			int children = monitorCmdList.getChildCount();
			for (int i=0; i<children; i++) {
				monitorCmdList.getChildAt(i).setVisibility(View.GONE);
			}
			
			if (monitorCmdsNum == 0) {
				TextView monItem = new TextView(context);
				monItem.setText(getString(R.string.strNoCmdsOver) + " " + 
						sp.getString("prefMonitorThreshold", "120") + " " + 
						getString(R.string.strSeconds));
				LayoutParams params = new LayoutParams();
				params.setMargins(5, 5, 5, 5);
				monItem.setLayoutParams(params);
				monitorCmdList.addView(monItem);
			}
			
			if (monitorCmdsNum == -1) {
				TextView monItem = new TextView(context);
				monItem.setText(getString(R.string.strNoMonitor));
				LayoutParams params = new LayoutParams();
				params.setMargins(5, 5, 5, 5);
				monItem.setLayoutParams(params);
				monitorCmdList.addView(monItem);
			}
			
			if (monitorCmdsNum == -2) {
				TextView monItem = new TextView(context);
				monItem.setText(getString(R.string.strSkippedNoUser));
				LayoutParams params = new LayoutParams();
				params.setMargins(5, 5, 5, 5);
				monItem.setLayoutParams(params);
				monitorCmdList.addView(monItem);
			}
			
			if (monitorCmdsNum == -3) {
				Button monItem = new Button(context);
				monItem.setText(getString(R.string.strNotLoggedIn));
				LayoutParams params = new LayoutParams();
				params.setMargins(5, 5, 5, 5);
				monItem.setLayoutParams(params);
				monItem.setOnClickListener(new OnClickListener() {
					@Override
					public void onClick(View view) {
						Intent loginIntent = new Intent(context, LoginDialog.class);
						loginIntent.putExtra(ID, serverId);
						loginIntent.putExtra(ERROR, PASSWORDINVALID);
						loginIntent.putExtra(CALLER, getLocalClassName());
			    		startActivity(loginIntent);
					}	
				});
				monitorCmdList.addView(monItem);
			}
			
			if (monitorCmdsNum == -4) {
				TextView monItem = new TextView(context);
				monItem.setText(getString(R.string.strNoSDCard));
				LayoutParams params = new LayoutParams();
				params.setMargins(5, 5, 5, 5);
				monItem.setLayoutParams(params);
				monitorCmdList.addView(monItem);
			}
			
			if (monitorCmdsNum > 0) {
				for (final IServerProcess proc : monitorCmds) {
					TextView monItem = new TextView(context);
					monItem.setText(String.valueOf(proc.getId()) + " " +
						    		proc.getStatus() + " " + proc.getTime() + " ( " +
						    		proc.getUserName() + " : " + proc.getCommand() +
						    		" )");
					LayoutParams params = new LayoutParams();
					params.setMargins(5, 5, 5, 5);
					monItem.setLayoutParams(params);
					monItem.setOnClickListener(new OnClickListener() {
						@Override
						public void onClick(View view) {
							if (runner[3] != null) {
								return;
							}
							
							class Worker extends Thread {
							    @Override
								public void run() {
									try {
										serverCaller.connect(true);
										final String emailAddr = serverCaller.getUser(proc.getUserName()).getEmail();
										if (Thread.interrupted()) {
											runner[3] = null;
											return;
										}
										if (emailAddr != null && emailAddr.length() > 2) {
											errorHandle.post(new Runnable() {
												@Override
												public void run() {
													Intent i = new Intent(android.content.Intent.ACTION_SEND);
													i.setType("plain/text");
													i.putExtra(android.content.Intent.EXTRA_EMAIL,
															new String[] {emailAddr});
													startActivity(Intent.createChooser(i, "Email user..."));
												}
											});
										}
								    	serverCaller.disconnect();
								    	runner[3] = null;
									} catch (Exception e) {
										// Lets just ignore this...
								    	runner[3] = null;
									}
							    }
							}
							
							runner[3] = new Worker();
							runner[3].start();
						}
					});
					monitorCmdList.addView(monItem);
				}
				((TextView) findViewById(R.id.tvWarnings))
					.setVisibility(View.VISIBLE);
				
				try {
					TextView warns = ((TextView) findViewById(R.id.tvWarningsText));
					warns.setVisibility(View.VISIBLE);
					warnStrLock.tryLock(120, TimeUnit.SECONDS);
					String warnsStr = warns.getText().toString();
					if (warnsStr.length() > 0) {
						warnsStr += "\n";
					}
					warns.setText(warnsStr + String.valueOf(monitorCmdsNum) + " " +
							getString(R.string.strWarnMonitor));
				} catch (InterruptedException e) {
					return;
				}
			}
		}
		
	};
	
    /**
     * Gets the "p4 info", stashes it and calls the info updater handle.
     * 
     * @see com.perforce.p4scout.ServerDetail#infoUpdateHandle
     */
    private void getInfo() {
		class Worker extends Thread {
		    @Override
			public void run() {
		    	try {
		    		serverCaller.connect(false);
				    info = serverCaller.getServerInfo();
					if (Thread.interrupted()) {
						return;
					}
				    infoUpdateHandle.post(infoUpdateUpdater);
				    serverCaller.disconnect();
				} catch (final Exception e) {
					if (Thread.interrupted()) {
						return;
					}
					errorHandle.post(new Runnable() {
						@Override
						public void run() {
							loading.dismiss();
							Toast.makeText(getApplicationContext(), e.getMessage().trim(),
									Toast.LENGTH_LONG).show();
							if (e.getMessage().trim().contains(PASSWORDINVALID) || 
									e.getMessage().trim().contains(SESSIONLOGGEDOUT) ||
					    			e.getMessage().trim().contains(PASSWORDNOTSET)) {
								Intent loginIntent = new Intent(context, LoginDialog.class);
								loginIntent.putExtra(ID, serverId);
								loginIntent.putExtra(ERROR, e.getMessage());
								loginIntent.putExtra(CALLER, getLocalClassName());
					    		startActivity(loginIntent);
							}
						}
					});
		    		    		
					e.printStackTrace();
					finish();
				}
		    }
		}
		runner[0] = new Worker();
		runner[0].start();
    }
    
    /**
     * Fetches a list of long running commands, stashes them and calls the monitor update handler.
     *
     * @see com.perforce.p4scout.ServerDetail#monitorHandle
     */
    private void getMonitor() {
    	final Context context = this;
		class Worker extends Thread {
		    @Override
			public void run() {
		    	try {
		    		monitorWorking = true;
		    		monitorCmdsNum = 0;
					// If user doesn't exist, we don't want to create it
				    if (!serverCaller.userExists()) {
				    	monitorCmdsNum = -2;
				    } else {
						if (Thread.interrupted()) {
							return;
						}
					    int monitorThreshold = Integer.parseInt(sp.getString("prefMonitorThreshold", "120"));
					    try {
						    List<IServerProcess> procs = serverCaller.getMonitor();
						    monitorCmdsNum = 0;
						    monitorCmds = new LinkedList<IServerProcess>();
						    for (IServerProcess proc : procs) {
								String t[] = proc.getTime().split(":");
								if ( ( (Integer.parseInt(t[0]) * 60 * 60) +
										(Integer.parseInt(t[1]) * 60) +
										Integer.parseInt(t[2]) ) > monitorThreshold ) {
									// Running time in seconds greater than threshold
									monitorCmds.add(proc);
									monitorCmdsNum++;
								}
							}
					    } catch (Exception re) {
					    	if (re.getMessage().trim().contentEquals(NOMONITOR)) {
					    		monitorCmdsNum = -1;
					    	} else if (re.getMessage().trim().contains(PASSWORDINVALID)
					    			|| re.getMessage().trim().contains(PASSWORDNOTSET)
					    			|| re.getMessage().trim().contains(SESSIONLOGGEDOUT)) {
					    		if (serverCaller.canUseLogin()) {
					    			monitorCmdsNum = -3;
					    		} else {
					    			monitorCmdsNum = -4;
					    		}
					    	} else {
				    			throw re;
				    		}
					    }
				    }
				    if (!userWorking) {
				    	serverCaller.disconnect();
				    }
					if (Thread.interrupted()) {
						return;
					}
				    monitorWorking = false;					
					monitorHandle.post(monitorUpdater);
				} catch (final Exception e) {
					if (Thread.interrupted()) {
						return;
					}
					errorHandle.post(new Runnable() {
						@Override
						public void run() {
							loading.dismiss();
							Toast.makeText(getApplicationContext(), e.getMessage().trim(),
									Toast.LENGTH_LONG).show();
							if (e.getMessage().trim().contains(PASSWORDINVALID) || 
									e.getMessage().trim().contains(SESSIONLOGGEDOUT) ||
					    			e.getMessage().trim().contains(PASSWORDNOTSET)) {
								Intent loginIntent = new Intent(context, LoginDialog.class);
								loginIntent.putExtra(ID, serverId);
								loginIntent.putExtra(ERROR, e.getMessage());
								loginIntent.putExtra(CALLER, getLocalClassName());
					    		startActivity(loginIntent);
							}
						}
					});
		    		    		
					e.printStackTrace();
					finish();
				}
		    }
		}
		runner[2] = new Worker();
		runner[2].start();
    }
    
    /**
     * Counts the number of users on the Perforce Server, stashes the number and calls the user bar
     * update handler.
     *
     * @see com.perforce.p4scout.ServerDetail#userCountHandle
     */
    private void getUsers() {
		class Worker extends Thread {
		    @Override
			public void run() {
		    	try {
		    		userWorking = true;
					serverCaller.connect(false);
					/*server.registerProgressCallback(new IProgressCallback(){
						// We have to override these, but they're not needed
						@Override
						public void start(int arg0) {
						}

						@Override
						public void stop(int arg0) {
						}
						// A tick is a user in this special case
						@Override
						public boolean tick(int arg0, String arg1) {
							userCount++;
							userCountHandle.post(userCountUpdater);
							return true;
						}
					});*/
					List<IUserSummary> userList = serverCaller.getUsers();
					if (!monitorWorking) {
						serverCaller.disconnect();
					}
					userWorking = false;
				    userCount = userList.size();
					if (Thread.interrupted()) {
						return;
					}
					userCountHandle.post(userCountUpdater);
				} catch (final Exception e) {
					if (Thread.interrupted()) {
						return;
					}
					errorHandle.post(new Runnable() {
						@Override
						public void run() {
							loading.dismiss();
							Toast.makeText(getApplicationContext(), e.getMessage().trim(),
									Toast.LENGTH_LONG).show();
							if (e.getMessage().trim().contains(PASSWORDINVALID) || 
									e.getMessage().trim().contains(SESSIONLOGGEDOUT) ||
					    			e.getMessage().trim().contains(PASSWORDNOTSET)) {
								Intent loginIntent = new Intent(context, LoginDialog.class);
								loginIntent.putExtra(ID, serverId);
								loginIntent.putExtra(ERROR, e.getMessage());
								loginIntent.putExtra(CALLER, getLocalClassName());
					    		startActivity(loginIntent);
							}
						}
					});
		    		    		
					e.printStackTrace();
					finish();
				}
		    }
		}
		runner[1] = new Worker();
		runner[1].start();
    }
    
    /**
     * This is the initialisation method for the Perforce Server infomation view.<br/>
	 * This displays the view and kicks off the Perforce commands in new threads.
	 * 
     * @see android.app.Activity#onCreate(android.os.Bundle)
     */
    @Override
    protected void onCreate(Bundle savedInstanceState) {
		super.onCreate(savedInstanceState);
		
		setContentView(R.layout.serverview);
		context = this;
		errorHandle = new Handler(Looper.getMainLooper());
		
		loading = new ProgressDialog(context);
		loading.setIndeterminate(true);
		loading.setOnCancelListener(new OnCancelListener() {
			@Override
			public void onCancel(DialogInterface arg0) {
				finish();
			}
		});
		loading.setMessage(getString(R.string.tvLoading));
		loading.show();
		
		Intent i = getIntent();
		Bundle extras = i.getExtras();
		
		runner = new Thread[4];
		
		serverId = (String) extras.get(ID);
		preUserCount = extras.getInt(USERS, -1);

		userLicenses = (TextView) findViewById(R.id.tvUserLicenses);
		monitorCmdList = (LinearLayout) findViewById(R.id.llMonitorList);

		((ScrollView) findViewById(R.id.svServerView)).setVisibility(View.GONE);
		
		sp = PreferenceManager.getDefaultSharedPreferences(context);
		
		serverCaller = new PerforceSession(this, serverId);

	    ((TextView) findViewById(R.id.tvServerAddress))
	    	.setText(serverCaller.getServerAddr());
	    setTitle(getString(R.string.strServerDetail) + " " + serverCaller.getServerName());
	    
		if (serverCaller.getServerDesc().length() > 0) {
			((TextView) findViewById(R.id.tvDescription))
				.setVisibility(View.VISIBLE);
			((TextView) findViewById(R.id.tvDescriptionText))
				.setText(serverCaller.getServerDesc());
			((TextView) findViewById(R.id.tvDescriptionText))
				.setVisibility(View.VISIBLE);
		}
    }
    
    /**
     * This is the method for inflating the option menu.
     * 
     * @see android.app.Activity#onCreateOptionsMenu(android.view.Menu)
     */
    @Override
    public boolean onCreateOptionsMenu(Menu menu) {
    	super.onCreateOptionsMenu(menu);
    	MenuInflater inflater = getMenuInflater();
    	inflater.inflate(R.menu.detailmenu, menu);
    	return true;
    }
    
    /**
     * This is the method for detecting which menu item was selected, and then deciding what to do
     * about it.
     * 
     * @see android.app.Activity#onOptionsItemSelected(android.view.MenuItem)
     */
    @Override
    public boolean onOptionsItemSelected(MenuItem item) {
    	switch (item.getItemId()) {
    	case R.id.editP4ServerMenu:
    		Intent editIntent = new Intent(this, ModifyServer.class);
    		editIntent.putExtra(ID, serverId);
    		editIntent.putExtra(ACTION, EDIT);
    		startActivity(editIntent);
    		return true;
	    case R.id.changesMenu:
			Intent changeIntent = new Intent(this, ChangesList.class);
    		changeIntent.putExtra(ID, serverId);
			startActivity(changeIntent);
			return true;
	    case R.id.jobsMenu:
	    	Intent jobsIntent = new Intent(this, JobList.class);
    		jobsIntent.putExtra(ID, serverId);
			startActivity(jobsIntent);
			return true;
		}
    	return true;
    }
    
    /**
     * This causes the monitor to be polled each time the page is refreshed
     */
    @Override
	protected void onResume() {
    	super.onResume();

    	// Clear any warnings
    	((TextView) findViewById(R.id.tvWarnings))
			.setVisibility(View.GONE);
		TextView warns = ((TextView) findViewById(R.id.tvWarningsText));
		warns.setVisibility(View.GONE);
		warns.setText("");
    	
    	
    	// Hide existing stuff first
		int children = monitorCmdList.getChildCount();
		for (int i=0; i<children; i++) {
			monitorCmdList.getChildAt(i).setVisibility(View.GONE);
		}
		
    	if (info != null) {
    		// Refresh potential license warnings
		    infoUpdateHandle.post(infoUpdateUpdater);
		    
		    // Continue to fetch users, or show warnings
    		if (userCount == 0) {
    			getUsers();
    		} else {
			    userCountHandle.post(userCountUpdater);
    		}
    		
    		// Refresh monitor output 
    		getMonitor();
		} else {
			// Start fetching stuff
			getInfo();
		}
	}

    /**
     * This kills all background threads when we exit. There is no need for them to consume
     * resources when we are doing something else.
     */
	@Override
    protected void onPause() {
    	super.onPause();
    	if (runner == null) {
    		return;
    	}
    	for (int i=0; i<runner.length; i++) {
    		if (runner[i] != null) {
    			runner[i].interrupt();
    		}
    	}
    }
	
	/**
	 * This prevents the activity from being restarted when the orientation changes
	 */
	@Override
	public void onConfigurationChanged(Configuration newConfig){        
	    super.onConfigurationChanged(newConfig);
	}

}
