/**
 * Copyright (c) 2010 Perforce Software.  All rights reserved.
 */
package com.perforce.p4scout;

import android.content.Context;

/**
 * This class defines a Perforce Server reference object.
 */
public class ServerInfo {
    
	/**
	 * The Checking status.
	 */
	public enum Checking {
        /** CONNECT - We're still trying to connect to the Perforce Server. */
        CONNECT, 
		/** WARNING - We're connected to the Perforce Server, but still checking for warnings. */
		WARNING, 
		/** COMPLETE - The status check is complete. */
		COMPLETE;
    }
	
	/**
	 * The Types of errors.
	 */
	public enum Errors {
        /** CONNECT - We're still trying to connect to the Perforce Server. */
        OFFLINE, 
		/** BADADDR - The Perforce Server address is not valid. */
		BADADDR, 
		/** RESOLVE - We can't resolve the hostname of the Perforce Server. */
		RESOLVE,
		/** UNKNOWN - There's an error, but we don't know what it is. */
		UNKNOWN,
		/** NONE - There are no errors. */
		NONE,
		;
    }
	
	/**
	 * The Status Enum.
	 */
	public enum Status {
        /** ONLINE - The Perforce Server is online and responding. */
        ONLINE,
		/** UNKNOWN - We don't know whether Perforce is up or down. */
		UNKNOWN, 
		/** WARNINGS - The Perforce Server is up, but has warnings. */
		WARNINGS, 
		/** ERRORS - There was an error connecting to the Perforce Server. */
		ERRORS;
    }
	
	/**
	 * The Warnings Enum.
	 */
	public enum Warning {
		/** USERS - There are less than the threshold of spare users. */
		USERS, 
		/** MONITOR - There are commands that have been running for longer than the specified time. */
		MONITOR, 
		/** LICENSE - The Perforce licence is close to expiry. */
		LICENSE;
	}
	
    public static final String ADDRESS = "address";
    public static final String NAME = "name";
    public static final String USED_LICENSES = "used";
    public static final String MAX_LICENSES = "max";
    public static final String P4USER = "p4user";
    public static final String P4PASSWD = "p4passwd";
	protected String idNum, name, desc, address, p4user, p4passwd;
    protected Status status;
    protected Checking checking;
    protected Errors error;
    protected int usedLicenses = 0;
    protected int maxLicenses = 0;
    protected int type = 0;
    protected int daysTilExpiry = 1000;
    protected int longRunningCmds = 0;
    protected boolean warnUsers = false;
    protected boolean warnMonitor = false;
    protected boolean warnLicense = false;
    protected Context context;
    
   
    /**
     * Instantiates a new Perforce Server reference object.
     *
     * @param context the parent class' context. (required)
     * @param name the name of the Perforce Server. (required)
     */
    public ServerInfo(Context context, String name) {
    	this.context = context;
    	this.name = name;
    }
    
    /**
     * Instantiates a new Perforce Server reference object.
     *
     * @param context the parent class' context. (required)
     * @param idNum the ID of the Perforce Server in the database. (required)
     * @param name the name of the Perforce Server. (required)
     * @param desc the Perforce Server description. (required)
     * @param address the address of the Perforce Server. (required)
     */
    public ServerInfo(Context context, String idNum, String name, String desc, String address) {
    	this.context = context;
    	this.idNum = idNum;
    	if (idNum == null) {
    		type = 1;
    	}
    	this.name = name;
    	this.address = address;
    	this.p4user = "nouser";
    	this.p4passwd = "";
    	this.desc = desc;
    	this.status = Status.UNKNOWN;
    	this.error = Errors.NONE;
    	this.checking = Checking.CONNECT;
    }
    
    /**
     * Instantiates a new Perforce Server reference object.
     *
     * @param context the parent class' context. (required)
     * @param idNum the ID of the Perforce Server in the database. (required)
     * @param name the name of the Perforce Server. (required)
     * @param desc the Perforce Server description. (required)
     * @param address the address of the Perforce Server. (required)
     * @param p4user the Perforce Server's users's UserName. (required)
     */
    public ServerInfo(Context context, String idNum, String name, String Desc, String address,
    		String p4user) {
    	this.context = context;
    	this.idNum = idNum;
    	if (idNum == null) {
    		type = 1;
    	}
    	this.name = name;
    	this.address = address;
    	this.p4user = p4user;
    	this.p4passwd = "";
    	this.status = Status.UNKNOWN;
    	this.error = Errors.NONE;
    	this.checking = Checking.CONNECT;
    }
    
    /**
     * Instantiates a new Perforce Server reference object.
     *
     * @param context the parent class' context. (required)
     * @param idNum the ID of the Perforce Server in the database. (required)
     * @param name the name of the Perforce Server. (required)
     * @param desc the Perforce Server description. (required)
     * @param address the address of the Perforce Server. (required)
     * @param p4user the Perforce Server's users's UserName. (required)
     * @param p4passwd the Perforce Server's users's Password. (required)
     */
    public ServerInfo(Context context, String idNum, String name, String desc, String address,
    		String p4user, String p4passwd) {
    	this.context = context;
    	this.idNum = idNum;
    	if (idNum == null) {
    		type = 1;
    	}
    	this.name = name;
    	this.desc = desc;
    	this.address = address;
    	this.p4user = p4user;
    	this.p4passwd = p4passwd;
    	this.status = Status.UNKNOWN;
    	this.error = Errors.NONE;
    	this.checking = Checking.CONNECT;
    }

    /**
     * Compare Perforce Server's names.
     *
     * @param si the Perforce Server reference object to compare to this one. (required)
     * @return true, if they match.
     */
    public boolean equals(ServerInfo si) {
    	return name.equals(si.getName());
    }
    
    /**
     * Compare Perforce Server's names.
     *
     * @param name the name of the Perforce Server to compare to this one. (required)
     * @return true, if they are identical.
     */
    public boolean equals(String name) {
    	return this.name.equals(name);
	}
    
    /**
     * Gets the address of the Perforce Server.
     *
     * @return the address of the Perforce Server
     */
    public String getAddress() { 
    	return address;
    }
    
    /**
     * Gets the checking status of the Perforce Server.
     *
     * @return the checking status of the Perforce Server.
     */
    public Checking getCheckStatus() { 
    	return checking;
    }
    
    /**
	 * Gets the description of the Perforce Server.
	 *
	 * @return the description of the Perforce Server.
	 */
	public String getDescription() {
		return desc;
	}    
    
    /**
     * Gets errors encountered when connecting to the Perforce Server.
     *
     * @return the error encountered.
     */
    public Errors getError() { 
    	return error;
    }
        
    /**
     * Gets a description of the errors encountered when connecting to the Perforce Server.
     *
     * @return the error encountered.
     */
    public String getErrors() {
    	switch (error) {
    	case NONE:
    		return "";
    	case UNKNOWN:
    		return context.getString(R.string.errUnknown);
		case BADADDR:
    		return context.getString(R.string.errBadAddr);
    	case RESOLVE:
    		return context.getString(R.string.errResolve);
    	case OFFLINE:
    		return context.getString(R.string.errOffline);
    	}
    	return null;
    }   

    /**
     * Gets the number of days until the Perforce license expire.
     *
     * @return the number of days until the license expires
     */
    public int getLicensesExpiresIn() {
    	return daysTilExpiry;
    }
    
    /**
     * Gets the number of long running cmds.
     *
     * @return the number of long running cmds.
     */
    public int getLongCmds() {
    	return longRunningCmds;
    }
        
    /**
     * Gets the total number of possible users.
     *
     * @return the total number of possible users.
     */
    public int getMaxLicenses() {
    	return maxLicenses;
    }  
    
    /**
     * Gets the name of the Perforce Server.
     *
     * @return the name of the Perforce Server
     */
    public String getName() {
    	return name;
    }
        
    /**
	 * Gets the Perforce user's password.
	 *
	 * @return the Perforce user's password.
	 */
	public String getPasswd() {
		return p4passwd;
	}
    
    /**
     * Gets the status of the Perforce Server.
     *
     * @return the status of the Perforce Server.
     */
    public Status getStatus() { 
    	return status;
    }
    
    /**
     * Gets the number of used user licenses.
     *
     * @return the number of used user licenses.
     */
    public int getUsedLicenses() {
    	return usedLicenses;
    }
    
    /**
	 * Gets the Perforce username.
	 *
	 * @return the Perforce username.
	 */
	public String getUser() {
		return p4user;
	}
    
    /**
	 * Gets the warnings on this Perforce Server as a String.
	 *
	 * @return the descriptive warning String.
	 */
	public String getWarnings() {
		String warnStr = "";
		if (warnUsers) {
			warnStr += String.valueOf(maxLicenses - usedLicenses) + " " +
				context.getString(R.string.strWarnUsers) + " ";
		}
		if (warnMonitor) {
			warnStr += String.valueOf(longRunningCmds) + " " + 
				context.getString(R.string.strWarnMonitor) + " ";
		}
		if (warnLicense && daysTilExpiry >= 0) {
			warnStr += String.valueOf(daysTilExpiry) + " " + 
				context.getString(R.string.strWarnLicense) + " ";
		} else if (warnLicense && daysTilExpiry == -1) {
			warnStr += context.getString(R.string.strNoMonitor) + " ";
		}
		return warnStr;
	}
    
    /**
     * Sets the address of the Perforce Server.
     *
     * @param address the address of the Perforce Server. (required)
     */
    public void setAddress(String address) {
    	this.address = address;
    }
    
    /**
     * Sets the checking status of the Perforce Server.
     *
     * @param checking the checking status of the Perforce Server. (required)
     */
    public void setCheckingStatus(Checking checking) {
    	if (checking == Checking.CONNECT) {
    		status = Status.UNKNOWN;
        	error = Errors.NONE;
    		warnUsers = false;
    		warnMonitor = false;
    		warnLicense = false;
    	}
    	this.checking = checking;
    }
    
    /**
     * Sets the error encountered when connecting to the Perforce Server.
     *
     * @param error the error encountered. (required)
     */
    public void setError(Errors error) {
    	this.error = error;
    }

	/**
     * Sets the license number of available and used Perforce licenses.
     *
     * @param used the number of users on the Perforce Server. (required)
     * @param max the number of users specified by the Perforce license. (required)
     */
    public void setLicenses(int used, int max) {
    	this.usedLicenses = used;
    	this.maxLicenses = max;
    }

	/**
     * Sets the number of days until the Perforce licenses expires.
     *
     * @param days until the license expires. (required)
     */
    public void setLicensesExpiresIn(int days) {
    	daysTilExpiry = days;
    }
    
    /**
     * Sets the number of long running cmds.
     *
     * @param cmds the number of long running cmds. (required)
     */
    public void setLongCmds(int cmds) {
    	longRunningCmds = cmds;
    }
    
    /**
     * Sets the password.
     *
     * @param p4passwd the new Perforce user's password. (required)
     */
    public void setPass(String p4passwd) {
    	this.p4passwd = p4passwd;
    }

	/**
     * Sets the status of the Perforce Server.
     *
     * @param status the Perforce Server's status. (required)
     */
    public void setStatus(Status status) {
    	this.status = status;
    }
	
	/**
     * Sets the user of the Perforce Server.
     *
     * @param p4user the Perforce user's UserName. (required)
     */
    public void setUser(String p4user) {
    	this.p4user = p4user;
    }
	
	/**
	 * Sets a Perforce Server warning.
	 *
	 * @param warn the type of warning to set.
	 */
	public void setWarning(Warning warn) {
		if (warn == Warning.USERS) {
			warnUsers = true;
		}
		if (warn == Warning.MONITOR) {
			warnMonitor = true;
		}
		if (warn == Warning.LICENSE) {
			warnLicense = true;
		}
	}
}
