/**
 * Copyright (c) 2010 Perforce Software.  All rights reserved.
 */
package com.perforce.p4scout;

import static android.provider.BaseColumns._ID;
import static com.perforce.p4scout.Constants.ACTION;
import static com.perforce.p4scout.Constants.DESC;
import static com.perforce.p4scout.Constants.EDIT;
import static com.perforce.p4scout.Constants.ID;
import static com.perforce.p4scout.Constants.NAME;
import static com.perforce.p4scout.Constants.P4PASS;
import static com.perforce.p4scout.Constants.P4PORT;
import static com.perforce.p4scout.Constants.P4SERVERS;
import static com.perforce.p4scout.Constants.P4USER;
import static com.perforce.p4scout.Constants.USERS;

import java.io.IOException;
import java.net.InetAddress;
import java.net.UnknownHostException;
import java.util.concurrent.locks.Lock;
import java.util.concurrent.locks.ReentrantLock;

import android.app.Activity;
import android.app.AlertDialog;
import android.content.Context;
import android.content.DialogInterface;
import android.content.Intent;
import android.content.SharedPreferences;
import android.content.DialogInterface.OnClickListener;
import android.content.DialogInterface.OnDismissListener;
import android.content.SharedPreferences.Editor;
import android.content.res.Configuration;
import android.database.Cursor;
import android.database.sqlite.SQLiteDatabase;
import android.os.Bundle;
import android.os.Handler;
import android.os.Message;
import android.preference.PreferenceManager;
import android.view.Menu;
import android.view.MenuInflater;
import android.view.MenuItem;
import android.view.View;
import android.widget.AdapterView;
import android.widget.ListView;
import android.widget.AdapterView.OnItemClickListener;
import android.widget.AdapterView.OnItemLongClickListener;

import com.perforce.p4scout.ServerInfo.Checking;
import com.perforce.p4scout.ServerInfo.Errors;
import com.perforce.p4scout.ServerInfo.Status;

/**
 * This class displays a list of Perforce Servers stored in a database.<br/>
 */
public class ServerList extends Activity {

    protected ServerListAdapter savedServerListAdapter;
  
    // DB setup
    private static String[] FROM = { _ID, NAME, DESC, P4PORT, P4USER, P4PASS };
    private static String ORDER_BY = NAME ;
    private P4Data storedServers;
	private Lock threadBlock;
	private int oneCount = 0;
	private int twoCount = 0;
	private SharedPreferences sp;
	private Thread[] runner;
    
    /** The Handler for updating the Perforce Server list in the View */
    private Handler savedServerHandle = new Handler();
    
    /** The Runnable for updating the Perforce Server list in the View. */
    private Runnable savedListUpdater = new Runnable() {
		public void run() {
			savedServerListAdapter.refresh();
			int items = savedServerListAdapter.getCount();
			if (items == 0) {
				findViewById(R.id.llNoServers).setVisibility(View.VISIBLE);
			} else {
				findViewById(R.id.llNoServers).setVisibility(View.GONE);
			}
		}
	};
    
    /**
     * This is the handler for ensuring that removed Perforce Servers are removed from the list.
     * */
    public Handler serverRemoved = new Handler() {
		@Override
		public void handleMessage(Message msg) {
			if (msg.obj != null) {
				int pos = savedServerListAdapter.findServer((String) msg.obj);
				if (pos != -1) {
					savedServerListAdapter.notifyRemoved(pos);
				}
    			runner = null;
    			runner = new Thread[savedServerListAdapter.getCount()];
		    }
		    savedServerListAdapter.notifyDataSetChanged();
		    savedServerHandle.post(savedListUpdater);
		}
    };
    
    /**
     * This checks the status of the Perforce Servers in new threads.
     *
     * @param i the position of the Perforce Server in the list. (required)
     */
    private void checkStatus(int i) {
    	final ServerInfo server = savedServerListAdapter.getInfo(i);
		
    	class Worker extends Thread {
		    @Override
			public void run() {
		    	server.setError(getInfo(server));
				if (Thread.interrupted()) {
					return;
				}
		    	if (server.getError() == Errors.NONE) {
		    		server.setStatus(Status.ONLINE);
					if (Thread.interrupted()) {
						return;
					}
			    	savedServerHandle.post(savedListUpdater);
			    	server.setStatus(getWarnings(server));
		    	} else if (server.getError() == Errors.UNKNOWN) {
		    		server.setStatus(Status.UNKNOWN);		    		
		    	} else {
		    		server.setStatus(Status.ERRORS);
		    	}
		    	server.setCheckingStatus(Checking.COMPLETE);
		    	savedServerHandle.post(savedListUpdater);
		    }
		}
		runner[i] = new Worker();
		runner[i].start();
    }

    /**
     * Checks whether or not we can connect to the Perforce Server.
     *
     * @param serverId the ID of the Perforce Server. (required)
     * @return the status of the Perforce Server
     * @throws InterruptedException 
     */
    private Errors getInfo(ServerInfo serverInfo) {
    	Errors rVal = Errors.UNKNOWN;
    	try {
	    	while (true) {
	    		threadBlock.lock();
	    		if (oneCount < Integer.parseInt(sp.getString("prefSimOps", "1"))) {
		    		break;
		    	}
	    		threadBlock.unlock();
	    		try {
					Thread.sleep(2000);
				} catch (Throwable e) {
					// We must have been interupted
					return Errors.UNKNOWN;
				}
			}
			oneCount++;
			threadBlock.unlock();
		
			//Can we connect?
			PerforceSession serverCaller = new PerforceSession(this, serverInfo.idNum);
			serverCaller.connect(false);
			serverCaller.disconnect();
		    //Yes

			rVal = Errors.NONE;
			
		} catch (Throwable e) {
			e.printStackTrace();
		    //No
			if (Thread.interrupted()) {
				return Errors.UNKNOWN;
			}
			try {
				//Can we ping the server?
				if (serverInfo != null && serverInfo.getAddress() != null
						&& serverInfo.getAddress().contains(":")) {
					InetAddress address = InetAddress.getByName(serverInfo.getAddress().split(":")[0]);
					address.isReachable(3000);
				} else {
					// The address is invalid
					rVal = Errors.BADADDR;
				}
				//Yes
				rVal = Errors.OFFLINE;
				
			} catch (UnknownHostException e1) {
				//No, because I can't resolve the name
				rVal = Errors.RESOLVE;
			} catch (IOException e2) {
				//No, because of something else
				rVal = Errors.UNKNOWN ;
			}
		} finally {
			threadBlock.lock();
			oneCount--;
			threadBlock.unlock();
		}
		return rVal;
    }
    
    
    /**
     * If we can connect to the Perforce Server, we can check for warnings.
     *
     * @param serverInfo the Perforce Server reference object. (required)
     * @return whether we have warnings or not.
     */
    private Status getWarnings(ServerInfo serverInfo){
		
		try {
			while (true) {
				threadBlock.lock();
				if (twoCount < Integer.parseInt(sp.getString("prefSimOps", "1"))) {
					break;
				}
				threadBlock.unlock();
				try {
					Thread.sleep(2000);
				} catch (Throwable e) {
					// We must have been interupted
					return Status.UNKNOWN;
				}
			}
			twoCount++;
			threadBlock.unlock();
			PerforceSession serverCaller = new PerforceSession(this, serverInfo.idNum);
			serverCaller.connect(false);
		    Status returnStatus = serverCaller.getWarnings(serverInfo);
		    serverCaller.disconnect();
		    return returnStatus;
		} catch (Throwable e) {
			e.printStackTrace();
		    return Status.UNKNOWN;
		} finally {
			threadBlock.lock();
			twoCount--;
			threadBlock.unlock();
		}
    }

    /**
     * This is the initialisation for the Perforce Server list view.<br/>
     * This mostly kicks of the UI loading.
     * 
     * @see android.app.Activity#onCreate(android.os.Bundle)
     */
    @Override
	public void onCreate(Bundle savedInstanceState) {
		super.onCreate(savedInstanceState);

		// Set initial variables and globals
		sp = PreferenceManager.getDefaultSharedPreferences(this);
		final Context context = this;
		
		// Set initial UI
		setContentView(R.layout.serverlist);
		setTitle(R.string.tvSavedServers);
		
		// Is this the first run?
		if (sp.getBoolean("prefFirstRun", true)){
			
			// Set up the license agreement.
			OnClickListener listener = new DialogInterface.OnClickListener() {
				@Override
				public void onClick(DialogInterface dialog, int which) {
					if (which == AlertDialog.BUTTON_POSITIVE) {
						Editor spEdit = sp.edit();
						spEdit.putBoolean("prefFirstRun", false);
						spEdit.commit();
						startActivity(new Intent(context, ServerList.class));
						finish();
					} else {
						finish();
					}
				}
			};
			
			AlertDialog licenseAccept = new AlertDialog.Builder(this).create();
			licenseAccept.setTitle(R.string.titleLicense);
			licenseAccept.setMessage(getString(R.string.tvAgreement) + "\n\n" +
							getString(R.string.tvDisclaimer));
			licenseAccept.setButton(AlertDialog.BUTTON_POSITIVE, getString(R.string.btAccept),
							listener);
			licenseAccept.setButton(AlertDialog.BUTTON_NEGATIVE, getString(R.string.btDecline),
							listener);
			licenseAccept.setCancelable(false);
			licenseAccept.setOnDismissListener(new OnDismissListener() {
				@Override
				public void onDismiss(DialogInterface dialog) {
					if (sp.getBoolean("prefFirstRun", true)) {
						finish();
					}
				}
			});
			
			licenseAccept.show();
			return;
		}
		
		
		// Initialize database
		storedServers = new P4Data(this);
		
		// Set thread handling stuff
		threadBlock = new ReentrantLock();		
		
		// Configure the list
		savedServerListAdapter = new ServerListAdapter(this);
	
		// Click listener works for both saved server and discovered server lists
		OnItemClickListener serverListClickListener = new OnItemClickListener() {
			public void onItemClick(AdapterView<?> parent, View view, int position, long id) {
	
				ServerListAdapter serverListAdapter = (ServerListAdapter) parent.getAdapter(); 
				ServerInfo serverInfo = serverListAdapter.getInfo(position);
		
				Intent i = new Intent(context, ServerDetail.class);
				i.putExtra(ID, serverInfo.idNum);
				if (serverInfo.getCheckStatus() == Checking.COMPLETE) {
					i.putExtra(USERS, serverInfo.getUsedLicenses());
				}				
				startActivity(i);
	
			}
		};
		
		// Click listener works for both saved server and discovered server
		// lists
		OnItemLongClickListener serverListLongClickListener = new OnItemLongClickListener() {
			public boolean onItemLongClick(AdapterView<?> parent, View view, int position, long id) {
	
				ServerListAdapter serverListAdapter = (ServerListAdapter) parent.getAdapter(); 
				final ServerInfo serverInfo = serverListAdapter.getInfo(position);
		
				new AlertDialog.Builder(context)
						.setTitle(serverInfo.getName())
						.setItems(R.array.ServerListOptions, new DialogInterface.OnClickListener() {
							public void onClick(DialogInterface dialog, int whichButton) {
								String[] items = getResources().getStringArray(R.array.ServerListOptions);
								if (items[whichButton].contentEquals(getString(R.string.optRemoveServer)))
								{
								    SQLiteDatabase db = storedServers.getReadableDatabase();
								    db.delete(P4SERVERS, _ID + "='" + serverInfo.idNum + "'", null);
								    serverRemoved.sendMessage(Message.obtain(serverRemoved, -1,
								    		new String(serverInfo.name)));
								    savedServerHandle.post(savedListUpdater);
								    db.close();
                            	}
								else if (items[whichButton].contentEquals(getString(R.string.optEditServer)))
								{
									Intent editIntent = new Intent(context, ModifyServer.class);
						    		editIntent.putExtra(ID, serverInfo.idNum);
						    		editIntent.putExtra(ACTION, EDIT);
						    		startActivity(editIntent);
								}
								else if (items[whichButton].contentEquals(getString(R.string.optJobSearch)))
								{
									Intent jobsIntent = new Intent(context, JobList.class);
						    		jobsIntent.putExtra(ID, serverInfo.idNum);
						    		startActivity(jobsIntent);
								}
								else if (items[whichButton].contentEquals(getString(R.string.optRecentChanges)))
								{
									Intent changesIntent = new Intent(context, ChangesList.class);
						    		changesIntent.putExtra(ID, serverInfo.idNum);
						    		startActivity(changesIntent);
								}
								else if (items[whichButton].contentEquals(getString(R.string.optGoto)))
								{
									Intent gotoIntent = new Intent(context, GotoDialog.class);
						    		gotoIntent.putExtra(ID, serverInfo.idNum);
						    		startActivity(gotoIntent);
								}
							}
                        })
                        .show();
				
				return true;
			}
		};
	
		ListView savedServers = (ListView) findViewById(R.id.lvSavedServers);
		savedServers.setAdapter(savedServerListAdapter);
		savedServers.setOnItemClickListener(serverListClickListener);
		savedServers.setOnItemLongClickListener(serverListLongClickListener);
	
    }
    
    /**
     * This is the method for inflating the option menu.
     * 
     * @see android.app.Activity#onCreateOptionsMenu(android.view.Menu)
     */
    @Override
    public boolean onCreateOptionsMenu(Menu menu) {
    	super.onCreateOptionsMenu(menu);
    	MenuInflater inflater = getMenuInflater();
    	inflater.inflate(R.menu.mainmenu, menu);
    	return true;
    }
    
    /**
     * This is the method for detecting which menu item was selected, and then deciding what to do
     * about it.
     * 
     * @see android.app.Activity#onOptionsItemSelected(android.view.MenuItem)
     */
    @Override
    public boolean onOptionsItemSelected(MenuItem item) {
    	switch (item.getItemId()) {
    	case R.id.addServer:
    		startActivity(new Intent(this, ModifyServer.class));
    		return true;
	    case R.id.aboutP4Android:
			startActivity(new Intent(this, ClientInfo.class));
			return true;
	    case R.id.refreshAll:
	    	int items = savedServerListAdapter.getCount();
	    	for (int i=0; i<items; i++) {
	    		if (savedServerListAdapter.getInfo(i).getCheckStatus() == Checking.COMPLETE) {
	    			savedServerListAdapter.getInfo(i).setCheckingStatus(Checking.CONNECT);
	    			savedServerListAdapter.notifyDataSetChanged();
	    			runner = null;
	    			runner = new Thread[items];
	    			checkStatus(i);
	    		}
	    	}
	    	return true;
	    case R.id.settingsGlobal:
			startActivity(new Intent(this, GlobalSettings.class));
			return true;
	    case R.id.showHelp:
			startActivity(new Intent(this, HelpDialog.class));
			return true;
		}
    	return true;
    }
    
    /**
     * We populate the list from the database and start checking their status' in new threads.
     * 
     * @see android.app.Activity#onResume()
     */
    @Override
    protected void onResume() {
    	super.onResume();
    	    	
		// Read saved servers file, build saved servers list
    	if (storedServers == null) {
    		return;
    	}
		SQLiteDatabase db = storedServers.getReadableDatabase();
	    Cursor cursor = db.query(P4SERVERS, FROM, null, null, null, null, ORDER_BY);
	    startManagingCursor(cursor);
	    int serverId = cursor.getColumnIndex(_ID);
	    int serverName = cursor.getColumnIndex(NAME);
	    int serverDesc = cursor.getColumnIndex(DESC);
	    int serverPort = cursor.getColumnIndex(P4PORT);
	    int serverUser = cursor.getColumnIndexOrThrow(P4USER);
	    int serverPasswd = cursor.getColumnIndexOrThrow(P4PASS);
	    
		while (cursor.moveToNext()) {
			int pos = savedServerListAdapter.getServer(cursor.getString(serverId));
			if (pos == -1) {
				savedServerListAdapter.notifyAdded(new ServerInfo(this, cursor.getString(serverId),
						cursor.getString(serverName), cursor.getString(serverDesc),
						cursor.getString(serverPort), cursor.getString(serverUser),
						cursor.getString(serverPasswd)));
			} else {
				savedServerListAdapter.notifyUpdated(new ServerInfo(this, cursor.getString(serverId),
						cursor.getString(serverName), cursor.getString(serverDesc),
						cursor.getString(serverPort), cursor.getString(serverUser),
						cursor.getString(serverPasswd)), pos);
			}
		}
	    cursor.close();
	    stopManagingCursor(cursor);
	    db.close();
		
		savedServerListAdapter.refresh();
    	
		int items = savedServerListAdapter.getCount();
		if (items == 0) {
			findViewById(R.id.llNoServers).setVisibility(View.VISIBLE);
		} else {
			findViewById(R.id.llNoServers).setVisibility(View.GONE);
		}
		runner = new Thread[items];
		oneCount = 0;
		twoCount = 0;
		for (int i =0; i < items; i++)
		{
			checkStatus(i);
		}
    }
    
    /**
     * This kills all background threads when we exit. There is no need for them to consume
     * resources when we are doing something else.
     */
    @Override
    protected void onPause() {
    	super.onPause();
    	
    	if (runner == null) {
    		return;
    	}
    	for (int i=0; i<runner.length; i++) {
    		if (runner[i] != null) {
    			runner[i].interrupt();
    		}
    	}
    }

	/**
	 * This prevents the activity from being restarted when the orientation changes
	 */
    @Override
    public void onConfigurationChanged(Configuration newConfig){        
        super.onConfigurationChanged(newConfig);
    }

}