/**
 * Copyright (c) 2010 Perforce Software.  All rights reserved.
 */
package com.perforce.p4scout;

import java.util.LinkedList;
import java.util.List;

import android.content.Context;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.widget.BaseAdapter;
import android.widget.ImageView;
import android.widget.LinearLayout;
import android.widget.TextView;

import com.perforce.p4scout.ServerInfo.Checking;

/**
 * This is the ListAdaptor implementation for displaying lists of Perforce Servers.
 * 
 * @see android.widget.BaseAdapter
 */
public class ServerListAdapter extends BaseAdapter {
    
    protected Context context;
    protected LayoutInflater inflater;
    protected List<ServerInfo> servers = new LinkedList<ServerInfo>();

    /**
     * Instantiates a new list adapter for Perforce Servers.
     *
     * @param context parent class' context
     */
    public ServerListAdapter(Context context) {
		this.context = context;
		this.inflater = (LayoutInflater) context
				.getSystemService(Context.LAYOUT_INFLATER_SERVICE);
    }
    
    /**
     * Find the postion of a Perfoce Server in the list.
     *
     * @param ident the name of the Perforce Server for identification. (required)
     * @return the postion of the Perforce Server in the list
     */
    public int findServer(String ident) {
		for (int i=0; i<servers.size(); i++) {
			if (servers.get(i).name.contentEquals(ident)) {
				return i;
			}
		}
		return -1;
    }
    
    /* (non-Javadoc)
     * @see android.widget.Adapter#getCount()
     */
    public int getCount() {
    	return servers.size();
    }
    
    /**
     * Gets the Perforce Server reference object by postion in the list.
     *
     * @param position the position of the Perforce Server in the list. (required)
     * @return the Perforce Server reference object.
     */
    public ServerInfo getInfo(int position) {
    	return servers.get(position);
    }
    
    /* (non-Javadoc)
     * @see android.widget.Adapter#getItem(int)
     */
    public String getItem(int position) {
    	return servers.get(position).getName();
    }
    
    /* (non-Javadoc)
     * @see android.widget.Adapter#getItemId(int)
     */
    public long getItemId(int position) {
    	return position;
    }
    
    /**
     * Gets the Perforce Server reference object by name.
     *
     * @param id the name of the Perforce Server. (required)
     * @return the Perforce Server reference object.
     */
    public int getServer(String id) {
		for (int i=0; i<servers.size(); i++) {
			if (servers.get(i).idNum.contentEquals(id)) {
				return i;
			}
		}
		return -1;
    }
    
    /** 
	 * This is the method that draws each Perforce Server item in the list.<br/>
	 * We change how we draw some elements based on errors, warnings and loading status.
	 * 
	 * @param position The position of the list item to be drawn 
	 * @param convertView 
	 * @param parent
	 * 
	 * @see android.widget.Adapter#getView(int, android.view.View, android.view.ViewGroup)
	 */
    public View getView(int position, View convertView, ViewGroup parent) {
	
		if (convertView == null) {
		    convertView = inflater.inflate(R.layout.serverlisting, parent, false);
		}
		
		try {
		    
		    ((TextView) convertView.findViewById(R.id.tvServerName))
				.setText(servers.get(position).getName());
		    
		    switch (servers.get(position).getStatus())
		    {
		    case ONLINE:
		    	((ImageView) convertView.findViewById(R.id.ivServerStatusIcon))
					.setImageResource(R.drawable.status_online);
		    	break;
		    case WARNINGS:
		    	((ImageView) convertView.findViewById(R.id.ivServerStatusIcon))
					.setImageResource(R.drawable.status_alert);
		    	break;
		    case ERRORS:
		    	((ImageView) convertView.findViewById(R.id.ivServerStatusIcon))
					.setImageResource(R.drawable.status_offline);
		    	break;
		    case UNKNOWN:
		    default:
		    	((ImageView) convertView.findViewById(R.id.ivServerStatusIcon))
					.setImageResource(R.drawable.status_unreachable);
		    	break;
		    }
		    
		    if (servers.get(position).getCheckStatus() == Checking.COMPLETE) {
		    	((LinearLayout) convertView.findViewById(R.id.llStillLoading))
					.setVisibility(View.GONE);
		    } else {
		    	((LinearLayout) convertView.findViewById(R.id.llStillLoading))
				.setVisibility(View.VISIBLE);
		    }
		    
		    if (servers.get(position).getWarnings().length() > 0) {
		    	
		    	TextView warnView = (TextView) convertView.findViewById(R.id.tvServerWarns);
		    	warnView.setVisibility(View.VISIBLE);
		    	warnView.setText(servers.get(position).getWarnings());
		    	
		    	((TextView) convertView.findViewById(R.id.tvServerDesc))
					.setVisibility(View.GONE);
		    	((TextView) convertView.findViewById(R.id.tvServerErrors))
					.setVisibility(View.GONE);
		    	
		    } else if (servers.get(position).getErrors().length() > 0) {
		    	
		    	TextView errorView = (TextView) convertView.findViewById(R.id.tvServerErrors);
		    	errorView.setVisibility(View.VISIBLE);
		    	errorView.setText(servers.get(position).getErrors());
		    	
		    	((TextView) convertView.findViewById(R.id.tvServerDesc))
					.setVisibility(View.GONE);
		    	((TextView) convertView.findViewById(R.id.tvServerWarns))
					.setVisibility(View.GONE);
		    	
		    } else {
		    	
		    	((TextView) convertView.findViewById(R.id.tvServerWarns))
					.setVisibility(View.GONE);
		    	((TextView) convertView.findViewById(R.id.tvServerErrors))
					.setVisibility(View.GONE);
		    	
		    	if (servers.get(position).getDescription().length() <= 0) {
			    	((TextView) convertView.findViewById(R.id.tvServerDesc))
						.setVisibility(View.GONE);
			    } else {
			    	TextView descView = (TextView) convertView.findViewById(R.id.tvServerDesc);
					descView.setVisibility(View.VISIBLE);
			    	descView.setText(servers.get(position).getDescription());
			    }
		    	
		    }
		}
		catch (Exception e) { 
		    e.printStackTrace();
		    ((TextView) convertView.findViewById(R.id.tvServerName))
				.setText(context.getString(R.string.strError));
		    ((ImageView) convertView.findViewById(R.id.ivServerStatusIcon))
				.setImageResource(R.drawable.status_unreachable);
	  
		}
		
		return convertView;
    }
    
    /**
     * Add a serverInfo object to the list of serverInfo.
     *
     * @param server reference to a Perforce Server. (required)
     */
    public void notifyAdded(ServerInfo server) {
    	servers.add(server);
    }

    /**
     * Removed the Perforcce Server from the list.
     *
     * @param position the Perforce Server in the list. (required)
     */
    public void notifyRemoved(int position) {
    	servers.remove(position);
    }
    
    /**
     * Replace the Perforce Server with an updated Perforce Server object.
     *
     * @param server the Perforce Server reference object. (required)
     */
    public void notifyUpdated(ServerInfo server) {
    	servers.remove(server);
    	servers.add(server);
    }

    /**
     * Replace the Perforce Server with an updated Perforce Server object.
     *
     * @param server the Perforce Server reference object. (required)
     * @param position the position of the Perforce Server to replace.
     */
    public void notifyUpdated(ServerInfo server, int position) {
    	servers.set(position, server);
    }

    /**
	 * The refresher method.
     */
    public void refresh() {
    	notifyDataSetChanged();
    }

}
